//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

public class PlotTrace implements Definitions {
  WinGroup wg; 
  
  int nfft;
  double t_samp;
  double tr[];
  
  double xmax, xmin;
  double ymax, ymin;
  double ydiff, meany;		// need it for plot
  double xdiff, meanx;		// need it for plot
  String theLabel;		// the label
  int id;
  
  int axis;			// shows x/y-axis-lin/log-plot
  String xLabel;
  boolean discr = false;	// if true plot true vertical lines
  
  int zoom_start = 0, zoom_end = 0, old_zoom_start = 0;
    
   PlotTrace ()
   {
	id = -1;
   }
   
   PlotTrace (PlotTrace orig, boolean sw)
   {
	nfft = orig.nfft;
	copyTraceData(orig.tr);
	t_samp = orig.t_samp;
	xLabel = orig.xLabel;
	axis = orig.axis;
	discr = orig.discr;
	zoom_start = orig.zoom_start;
	zoom_end = orig.zoom_end;
	old_zoom_start = orig.old_zoom_start;
	theLabel = orig.theLabel;
	id = orig.id;
	if (sw)
	    MaxMin();
	else
	{
	    xmax = orig.xmax; xmin = orig.xmin;
	    ymax = orig.ymax; ymin = orig.ymin;
	    xdiff = orig.xdiff; meanx = orig.meanx;
	    ydiff = orig.ydiff; meany = orig.meany;
	}
    }
    
    PlotTrace (PlotTrace orig, int ndat)
    {
      int i, ncopy;
	nfft = ndat;
	t_samp = orig.t_samp;
	tr = new double [nfft];
	ncopy = (orig.nfft < ndat) ? orig.nfft : ndat; 
	for (i = 0; i < ncopy;i++)
	    tr[i] = orig.tr[i];
	for (i = ncopy; i < nfft; i++)
	    tr[i] = 0;
	
	realft(tr,nfft/2,1,nfft/2);
	for (i = 0; i < nfft; i++)
	    tr[i] *= t_samp; 
    }
    public int getId ()		{ return id; }
    public void setId (int i)	{ id = i; }
      
    public void copyTraceData (double [] data)
    {
	tr = new double [data.length];
	for (int i = 0; i < data.length; i++) 
	    tr[i] = data[i];
    }
    	    
    public void ZoomTrace(int start, int end)
    {
      int i;
	if (end < start)
	{
	    i = end;
	    end = start;
	    start = i;
	} 
	if (start < 0) start = 0;
	if (end > nfft) end = nfft;
	if (end < 0 || start > nfft) return;
	double t[] = new double [end-start];
	for (i = start; i < end; i++)
	    t[i-start] = tr[i];
	tr = t;
	nfft = end-start;
	old_zoom_start += zoom_start;
	zoom_start = start;
	zoom_end = end;
	MaxMin();
    }
    	    
    public void ZoomTrace (double start, double end)
    {
      int i;
      	old_zoom_start += zoom_start;
	zoom_start = (int)(start/t_samp) - old_zoom_start;
	zoom_end =  zoom_start + (int)((end - start)/t_samp);
	nfft = (int)((end - start)/t_samp);
	double t[] = new double [nfft];
	for (i = zoom_start; i < zoom_end; i++)
	    t[i-zoom_start] = tr[i];
	tr = t;
	MaxMin();
    }	
    
    public void VerticalZoom()
    {
	ymax = ymax/2;
	ymin = ymin/2;
	for (int i = 0; i < nfft; i++)
	{
	    if (tr[i] > ymax) tr[i] = ymax;
	    if (tr[i] < ymin) tr[i] = ymin;
	}
	MaxMin();
    }
   
    public double getXmin()
    {
	double min = (zoom_start != 0) ? zoom_start*t_samp : 0;
	if (old_zoom_start != 0)
	    min += old_zoom_start*t_samp;
	return min;
    }
    public double getXmax()
    {
	double max = (zoom_end != 0) ? zoom_end*t_samp : nfft*t_samp;
	if (old_zoom_start != 0)
	    max += old_zoom_start*t_samp;
	return max;
    }  
    public void MaxMin ()
    {
      int i = 0;
	xmin = getXmin();
	xmax = getXmax();
	if (axis == 2 || axis == 3)	    // xlog
	{
	    while (xmin <= 0)
		xmin = ++i * t_samp;
	    xmin = Math.floor(0.4343*Math.log(xmin));
	    xmax = Math.ceil(0.4343*Math.log(xmax));
	}
	if (axis == 0 || axis == 2)	    // ylin
	{
	    ymin = Ymin();
	    ymax = Ymax();
	}
	if (axis == 1 || axis == 3)	    // ylog
	{
	    ymin = Math.floor(0.4343*Math.log(nzYmin()));
	    ymax = Math.ceil(0.4343*Math.log(nzYmax()));
	}
	if (ymin == ymax)
	{
	    ymin -= 1;
	    ymax += 1;
	}  
	ydiff = ymax - ymin;
	meany = (ymax + ymin)/2;
	xdiff = xmax - xmin;
	meanx = (xmax + xmin)/2;
    }
    
    public void SameMaxMin (PlotTrace t)
    {
	double max = (ymax > t.ymax) ? ymax : t.ymax;
	double min = (ymin < t.ymin) ? ymin : t.ymin;
	ymax = t.ymax = max;
	ymin = t.ymin = min;
	ydiff = t.ydiff = max - min;
	meany = t.meany = (max + min)/2;
     }
    
    private double Ymin()
    {
      double min;
	min = tr[0];
	for (int i = 1; i <nfft; i++)
	    if (tr[i] < min)
		min = tr[i];
	return min;
     }	
  
    private double nzYmin()
    {
      double min = 1e37;;
	for (int i = 0; i < nfft; i++)
	    if (tr[i] > 0.0 && tr[i] < min)
		min = tr[i];
	return min;
    }
  
    private double Ymax()
    {
      double max;
	max = tr[0];
	for (int i = 1; i < nfft; i++)
	    if (tr[i] > max)
		max = tr[i];
	return max;
     }	
  
    private double nzYmax()
    {
      double max = -1e37;;
	for (int i = 0; i <nfft; i++)
	    if (tr[i] > 0.0 && tr[i] > max)
		max = tr[i];
	return max;
    }
  
  public int convolve (int decratio)
  {
    double trdc[],tr1[],tr2[], tr_new[];
    int i,k;
    int ndat1,ndat2;
    String url_file;
	url_file = wg.tutor.codeBase+wg.Dir_decFIR+wg.File_decFIR+decratio;
	trdc = wg.ServerConnection.ReadDataFile(url_file);
	ndat1 = (trdc.length > tr.length) ? trdc.length : tr.length;
	tr1 =   (trdc.length > tr.length) ? trdc : tr;
	ndat2 = (trdc.length > tr.length) ? tr.length : trdc.length;
	tr2 =   (trdc.length > tr.length) ? tr : trdc;

	nfft = ndat1 + (ndat2-1); 
	tr_new = new double [nfft];
	
	/* convolve */
	for (i = 0; i < nfft; i++)
	{
	    tr_new[i] = 0.0;
	    /* MA */
	    for (k = 0; k < ndat2; k++)
                if(( i - k >= 0) && (i - k < ndat1))
                    tr_new[i] += tr2[k]*tr1[i-k];
        }
	copyTraceData (tr_new);
	return (trdc.length);
    }	

    // Barplot
    public void barplot (double t_samp_new, double t_samp_old, int ndat)
    {
      int j, k, jj, win;
      double tr2[];
     
     	tr2 = new double[ndat];
	for (j = 0; j < nfft;j++)
	{
	    k = (int)(((double)j)/(t_samp_old/t_samp_new));      
	    win = (int)(t_samp_new/(2*t_samp_old));
	    for(jj = k-win;jj<=k+win;jj++)
	    {
		if(jj >= 0 && jj < ndat)
		    tr2[jj] = tr[j]; 
	    } 
	}
	copyTraceData(tr2);
	nfft = ndat;
    }
   
   // Discretize
   public void discretize(double t_samp_new, double t_samp_old, int ndat)
   {
	int j, k, jj, win;
	double tr2[];
     
     	tr2 = new double[ndat];
	for (j = 0; j < nfft;j++)
	{
	    k = (int)(((double)j)/(t_samp_old/t_samp_new));      
	    win = 0;
	    for(jj = k-win;jj<=k+win;jj++)
	    {
		if(jj >= 0 && jj < ndat)
		    tr2[jj] = tr[j]; 
	    }
	}     
	tr = tr2;
	nfft = ndat;
    }
   
    // Resampling	 	
    public double[] resample (double t_samp_old,double t_samp_new,int ntake,double skip)
    {
      // t_samp_old - sampling interval used for generation of input trace
      double skip_rest;
      int nskip;
      double x1,old_x, new_x;
      double old_y_low, old_y_high;
      int closest_sample, int_start_sample = 0;
      int old_low;
      int ndat, ndat2;
      double tr1[], tr2[],return_tr[];
      int j,k,jj;
	nskip = (int)Math.floor(skip);
	skip_rest = skip - nskip;
	ndat = nfft - nskip;

	x1 = t_samp_old/t_samp_new;
	x1 *= ndat;
	ndat2 = (int)Math.ceil(x1);
	// don't allow for interpolation
	if(t_samp_new <= t_samp_old)
	{
	    t_samp_new = t_samp_old;
	    ndat2 = ndat;
	}
	tr1 = new double [ndat];
	tr2 = new double [ndat];
	
	for(k = 0; k < ndat+nskip; k++)
	    if(k >= nskip)
		tr1[k-nskip] = tr[k];
	for (j = 0; j < ndat2;j++)
	{
	    /* new interpolation -> spline interpolation */
	    new_x = j*t_samp_new + skip_rest*t_samp_old;
	    /* determine index of closest sample in trace */
	    closest_sample = (int)(new_x/t_samp_old);
 
	    if ((closest_sample >= (int)((double)n_int/2.)) &&
		(closest_sample <= ndat - (int)((double)n_int/2.) - 1))
	    {
		int_start_sample = closest_sample - (int)((double)n_int/2.) - 1;
	    }
	    else if (closest_sample < (int)((double)n_int/2.))
            {
               int_start_sample = -1;
            }
            else if (closest_sample > ndat - (int)((double)n_int/2.) - 1)
            {
               int_start_sample = ndat -1 - n_int;
            }
 
            for (jj = 1;jj <= n_int; jj++)
            {
              ya[jj] = tr1[int_start_sample + jj];
              xa[jj] = (int_start_sample + jj)*t_samp_old;
            }
            x1 = polint(new_x);
            tr2[j] = x1;
	}
	
	if(ntake <= 0 || ntake > ndat2) ntake = ndat2;
	return_tr = new double [ndat];
        for(k = 0; k < ntake; k++)
	    return_tr[k] = tr2[k];
	nfft = ntake;
	t_samp = t_samp_new; 
	return(return_tr);    
  }
    
  double xa[] = new double[5];		// values for ADC
  double ya[] = new double[5];
  double delta_value;
  int gain;
  int n_int = 4; /* degree of polynomial for interpoltion */
      
    long pow_I(int mant, int expo)
    {
	long i,dummy;
	if(expo == 0) return(1L);
	if(expo < 0) return(0L);
	dummy=1L;
	for(i=1;i<= expo;i++)
	    dummy *= mant;
	return(dummy);
    }
    
       
  double quantize(double inp,int bits, double lsb, double gain_up, long no_gains)
  {
     int old_gain = 0;
     int low_gain_count = 0;
     long i;

     long out_states; /* no. of output states of the quantizer */
     double out_val;
     double gain_fac;
     int base;

     base = 2;

     /* reset */
     if(no_gains == 1) {
         old_gain = 0;
         low_gain_count = 0;
     }
     gain = old_gain;
     out_states = pow_I(base,bits);
     gain_fac = pow_I(2,(gain));

     while ((inp/gain_fac > lsb*out_states) && ( gain < (no_gains - 1) ))
     {
         gain++;
         gain_fac = pow_I(2,gain);
     }


     if((inp/gain_fac <= lsb*out_states*(gain_up)))
     {
         low_gain_count++;
         if(low_gain_count > 0)
         {
              low_gain_count = 0;
              gain--;
              if (gain < 0) gain = 0;
              gain_fac = pow_I(2,gain);
         }
     }

     i = 0;
     while (( inp/gain_fac > (lsb/2 + (lsb * i))) && ( i < out_states - 1 ))
          i++;

     out_val = lsb * i;

     old_gain = gain;
     return (out_val);
  }
  
  public double polint(double x)
  {
	int i, m, ns=1;
	double den,dif,dift,ho,hp,w;
	double c[] = new double[n_int+1];
	double d[] = new double[n_int+1];
	double y;

	dif = Math.abs(x-xa[1]);
	for (i = 1; i <= n_int; i++) {
		if ( (dift = Math.abs(x-xa[i])) < dif) {
			ns=i;
			dif=dift;
		}
		c[i]=ya[i];
		d[i]=ya[i];
	}
	y=ya[ns--];
	for (m = 1; m < n_int; m++) {
	    for (i = 1; i <= n_int-m; i++) {
		ho=xa[i]-x;
		hp=xa[i+m]-x;
		w=c[i+1]-d[i];
		den = ho-hp;
		//if ( (den=ho-hp) == 0.0) nrerror("Error in routine POLINT");
		den=w/den;
		d[i]=hp*den;
		c[i]=ho*den;
	    }
	    delta_value =(2*ns < (n_int-m) ? c[ns+1] : d[ns--]);
	    y += delta_value;
	}
	return (y);
  }
  
  void realft(double da[], int n, int isign, int nfour)
  {
        int i,i1,i2,i3,i4,n2p3;
        double c1=0.5,c2,h1r,h1i,h2r,h2i;
        double wr,wi,wpr,wpi,wtemp,theta;
		
	double data[] = new double[nfft+1];
	for (i = 0; i <nfft; i++)
	    data[i+1] = da[i];
   
        theta = Math.PI/(double) nfour;
        if (isign == 1) {
                c2 = -0.5;
                four1(data,nfour,1);
        } else {
                c2=0.5;
                theta = -theta;
        }
        wtemp = Math.sin(0.5*theta);
        wpr = -2.0*wtemp*wtemp;
        wpi = Math.sin(theta);
        wr = 1.0 + wpr;
        wi = wpi;
        n2p3 = 2*nfour+3;
	for (i=2;i<=nfour/2;i++) {
                i4=1+(i3=n2p3-(i2=1+(i1=i+i-1)));
                h1r=c1*(data[i1]+data[i3]);
                h1i=c1*(data[i2]-data[i4]);
                h2r = -c2*(data[i2]+data[i4]);
                h2i=c2*(data[i1]-data[i3]);
                data[i1]=h1r+wr*h2r-wi*h2i;
                data[i2]=h1i+wr*h2i+wi*h2r;
                data[i3]=h1r-wr*h2r+wi*h2i;
                data[i4] = -h1i+wr*h2i+wi*h2r;
                wr=(wtemp=wr)*wpr-wi*wpi+wr;
                wi=wi*wpr+wtemp*wpi+wi;
        }
        if (isign == 1) {
                data[1] = (h1r=data[1])+data[2];
                data[2] = h1r-data[2];
        } else {
                data[1]=c1*((h1r=data[1])+data[2]);
                data[2]=c1*(h1r-data[2]);
                four1(data,nfour,-1);
	}
	for (i = 0; i <nfft; i++)
	    da[i] = data[i+1];
  }

  void four1(double data[], int nn, int isign)
  {
        int n,mmax,m,j,istep,i;
        double wtemp,wr,wpr,wpi,wi,theta;
        double tempr,tempi;
        n = nn << 1;
	j = 1;
	for (i=1;i<n;i+=2) {
                if (j > i) {
			tempr = data[j];
			data[j] = data[i];
			data[i] = tempr;
			tempr = data[j+1];
			data[j+1] = data[i+1];
			data[i+1] = tempr;
                }
                m=n >> 1;
                while (m >= 2 && j > m) {
                        j -= m;
                        m >>= 1;
                }
                j += m; 
        }
	mmax=2;
        while (n > mmax) {
                istep = 2*mmax;
                theta= (2*Math.PI)/(isign*mmax);
                wtemp = Math.sin(0.5*theta);
                wpr = -2.0*wtemp*wtemp;
                wpi = Math.sin(theta);
                wr=1.0;
                wi=0.0;
	        for (m=1;m<mmax;m+=2) {
                      for (i=m;i<=n;i+=istep) {
		                j=i+mmax;
                                tempr=wr*data[j]-wi*data[j+1];
                                tempi=wr*data[j+1]+wi*data[j];
                                data[j]=data[i]-tempr;
                                data[j+1]=data[i+1]-tempi;
                                data[i] += tempr;
                                data[i+1] += tempi;
		        }
                        wr=(wtemp=wr)*wpr-wi*wpi+wr;
                        wi=wi*wpr+wtemp*wpi+wi;
                }
                mmax=istep;
	}	
  }
  
  public void spr_specmult(double tr1[], double tr2[])
  {
    int i;                      
    double real, imag;

    tr1[0] = tr2[0];
    tr1[1] = tr2[1];

    for (i=1; i < nfft/2; i++)
    {
        real = tr1[2*i]*tr2[2*i] - tr1[2*i+1]*tr2[2*i+1];
        imag = tr1[2*i]*tr2[2*i+1] + tr1[2*i+1]*tr2[2*i];
        tr1[2*i] = real;
        tr1[2*i+1] = imag;
    }
  }
}
