//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.*;

public class ProceedSignal extends PlotTrace{

 
  boolean proceeding_done = false;
  
    public ProceedSignal (WinGroup w)
    {
	wg = w;
	axis = 0;
	xLabel = "s";
    }
    
    public ProceedSignal (WinGroup w, PlotTrace t, int param)
    {
	wg = w;
	axis = 0;
	xLabel = "s";
	discr = false;
	id = param;
	t_samp = t.t_samp;
	nfft = t.nfft;
	copyTraceData (t.tr);

	switch (id) {
	    case MOVINGAVERAGE:		theLabel = movingAverageSt; break;
	    case INTERPOLATE:		theLabel = interpolateSt; break;
	    case DISCRETIZATION:	theLabel = discretizationTitelSt; break;
	    case RECONSTRUCTION:	theLabel = reconstructionSt; break;
	    case DECIMATE:		theLabel = decimateSt; break;
	    case INTEGRATION:		theLabel = integrationSt; break;
	    case PLAINSUMMATION:	theLabel = plainSummationSt; break;
	    case DIFFERENTIATE:		theLabel = differentiateSt; break;
	    case PLAINADCQUANTIZED:	theLabel = plainADCSt+": "+
			plainADCQuantizedSt.substring(0,plainADCQuantizedSt.lastIndexOf("(")); break;
	    case PLAINADCMANTISSA:	theLabel = plainADCSt+": "+plainADCMantissaSt; break;
	    case PLAINADCEXPONENT:	theLabel = plainADCSt+": "+
			plainADCExponentSt.substring(0,plainADCExponentSt.lastIndexOf("(")); break;
	    case PLAINADCERROR:		theLabel = plainADCSt+": "+plainADCErrorSt; break;
	    case DECIMATIONFILTERING:	theLabel = decimationFilteringSt; break;
	    case FIR2CAUS:		theLabel = fir2causSt; break;
  	    case DIFFERENCEEQUATION:	theLabel = differenceEquationSt; break;
	    case FILTERWITHGSECALIB:	theLabel = filterWithGseCalibSt; break;
	    case BUTTERWORTH_BP:	theLabel = butterworthSt+butterworthBPSt; break;
	    case BUTTERWORTH_LP:	theLabel = butterworthSt+butterworthLPSt; break;
	    case BUTTERWORTH_HP:	theLabel = butterworthSt+butterworthHPSt; break;
	    case DELTAMODULATION:	theLabel = deltaModulationSt; break;
	    case SIGMADELTAMODULATION:	theLabel = sigmaDeltaModulationSt; break;
	    case IMPULSERESPONSE:	theLabel = impulseResponseSt; break;
	    case STEPRESPONSE:		theLabel = stepResponseSt; break;
	}
    }
    
 //
 // ---------- Integrate ---------
 //
    public void MakeIntegrate ()
    {
	copyTraceData (time_int (tr, nfft, t_samp));
    }
    
    public double [] time_int(double tr1[], int ndat, double tsamp)
    {
	int j;
	double y[];
	y  = new double[ndat];
	y[0] = tr1[0]*tsamp/2;
	for (j = 1; j < ndat-1;j++)
	{
	    y[j] = y[j-1] + tr1[j]*tsamp;
	}
	y[ndat-1] = y[ndat-2] + tr1[ndat-1]*tsamp/2;
	return (y);
    } 
    
 //
 // ---------- Differentiate -----
 //
    public void Differentiate ()
    {
      int i;
      double buff;
      double buff0;

        buff0 = tr[0];
        tr[0] *= 2/t_samp;
        for (i = 1; i < nfft; i++)
        {
            buff = tr[i];
            tr[i] = (tr[i] - buff0)/t_samp;
            buff0 = buff;
        }
    }
 
 //   
 // ---------- GSE Response ----------
 //
  public void BuildGSEResponseTrace (int n, double spec[])
  {	
    int i,ndat,ncopy;
       // save the original data
	ndat = nfft;
	double trace[] = new double [nfft];
	for (i = 0; i < nfft; i++)
	    trace[i] = tr[i];
	
	tr = new double [(n > nfft) ? n : nfft];
	ncopy = (n > nfft) ? nfft : n;
	for (i = 0; i < ncopy; i++)
	    tr[i] = trace[i];
	
	nfft = n;
	realft (tr,n/2,1,n/2); 
	spr_specmult (tr, spec);
	realft (tr,n/2,-1,n/2); 
	for(i = ncopy; i < nfft;i++)
	    tr[i] = 0.0;
        for(i = 0; i < nfft; i++)
	    tr[i] /= nfft/2;
	nfft = ndat;
  } 
  
  //
  // ---------- Decimation filtering ----------
  //
    public void DecimationFiltering (int decratio, String resamp)
    {
      int nfilt, ntake;
      
	ntake = (int)Math.ceil((double)nfft/(double)decratio);
	nfilt = convolve (decratio);
	if (resamp.equals (decimationFilteringResampOnSt))
	    copyTraceData (resample(t_samp, t_samp*decratio, ntake,(nfilt-1)/2.0));
    }
     
  // 
  // ---------- Decimate ----------
  //
    public void Decimation(int dec_ratio, int pos)
    {
      int max_pos;
      int j;	
      double max;
      int ndat,ndat2;
      double x[], tr1[];
	t_samp *= dec_ratio;
	 
	ndat = nfft;
	max = Math.abs(tr[0]);
	max_pos = 0;
	for (j = 0; j < ndat;j++)
	{
	    if(Math.abs(tr[j]) > max){
		max = Math.abs(tr[j]);
		max_pos = j;
	    }
	}
	// only for negative start positions take the position of the maximum 
	if (pos >= 0) max_pos = pos;  
  
	ndat2 = ndat/dec_ratio;
	x  = new double [ndat2+1];
	for (j = max_pos; j < ndat;j += dec_ratio)
	{
	    if((j/dec_ratio < ndat2) && (j/dec_ratio>=0))
		x[j/dec_ratio] = tr[j];
	}
	for (j = max_pos - dec_ratio; j >= 0;j -= dec_ratio)
	{
	    if((j/dec_ratio < ndat2) && (j/dec_ratio>=0))
		x[j/dec_ratio] = tr[j];
	}
	nfft = ndat2;
   	tr = new double [nfft];
	for (j = 0; j < ndat2;j++)
	    tr[j] = x[j];
   }
    
    //
    // ---------- Moving Average ----------
    //
    public void MovingAverage(double win_length)
    {
      int half_width;
	half_width = (int)(win_length/(2.0*t_samp));
	if(half_width < 0 ) half_width = 1;
        if(half_width > (nfft/2 - 1)) half_width = nfft/2 - 1;
	copyTraceData(spr_run_av (tr, half_width));
    }
        public double [] spr_run_av (double tr1[], int nx2)
    {	
      int i,j,k = 0;
      double sum1,sum2;
      double tr2[], tr3[];
	tr2 = new double [nfft];
	tr3 = new double [nfft];
        for(i = nx2; i <= nfft-(nx2+1); i++) {
	    sum1 = sum2 = 0.0;
	    for(j = 0; j <= nx2; j++) {
                sum1 += tr1[i-j];
                sum2 += tr1[i+j];
            }
            tr2[i] = (sum1+sum2+tr1[i])/(double)(2*nx2+2);
	}
        for(i = 1; i <= nx2-1; i++) {
            sum1 = sum2 = 0.0;
            for(j = 1; j <= i; j++) {
		sum1 += tr1[i-j];
		sum2 += tr1[i+j];
		k=j;
            }
            tr2[i] = (sum1+sum2+tr1[i])/(double)(2*k+2);
        }
	for(i = 1; i <= nx2-1; i++) {
             sum1 = sum2 = 0.0;
             for(j = 1; j <= i; j++) {
		sum1 += tr1[nfft-1-i-j];
		sum2 += tr1[nfft-1-i+j];
		k=j;
             }
             tr2[nfft-1-i] = (sum1+sum2+tr1[nfft-1-i])/(double)(2*k+2);
        }

        tr2[nfft-1] = tr1[nfft-1];
        tr2[0] = tr1[0];
	for(i = 0; i <= nfft-1; i++)
            tr3[i] = tr1[i];

        for(i = 0; i <= nfft-1; i++)
            tr3[i] -= tr2[i];
	return (tr2);
    }	
    
    //
    // ---------- Diffrence Equation ----------
    //
    public void DifferenceEquation(double a[], double b[], double gain, boolean inv)
    { 
       int i,l,k,m,n;
       double MAXFLOAT = 1e37;
       double u[],y[];
	
	u = new double[nfft];
	y = new double[nfft];
	n = a.length;
	m = b.length;
	
	for (i = 0; i < nfft; i++)
	{
	    if (!inv) u[i] = tr[i];
	    else      u[i] = tr[nfft-1-i];
	}    
	for (i = 0; i < nfft; i++)
	{    
	    y[i] = 0.;
	    
	    /* MA */
	    for (l = 0; l <= m-1; l++)
	    	if ((i-l) >= 0)
		    y[i] += u[i-l]*b[l]/a[0];
	    /* AR */
	    for (k = 1; k <= n-1; k++)
		if ((i-k) >= 0)
		{
		    y[i] -= y[i-k]*a[k]/a[0];
		    if (y[i] > MAXFLOAT/100) y[i] = MAXFLOAT/100;
		    if (y[i] < -MAXFLOAT/100) y[i] = -MAXFLOAT/100;
		}
	}
	for (i = 0; i < nfft; i++)
	    if (!inv) tr[i] = y[i] * gain;
	    else      tr[i] = y[nfft-1-i] * gain;
    }	
    
    //
    // ---------- discretization ---------- 
    //
    public void discretisation (double freq)
    {
      int old_low; 
      double old_x, new_x, old_y_low, old_y_high;	
      double t_samp_old, t_samp_new;
      double x1;
      int i, k;
      int ndat2;
      
	discr = true;
	
	t_samp_old = t_samp;
        t_samp_new = 1.0/freq;
        x1 = t_samp_old/t_samp_new;
	x1 *= nfft;
        ndat2 = (int)Math.ceil(x1);
	
	double t[] = new double[nfft];
        for (i = 0; i < nfft; i++)
	    t[i] = 0.0;
        for (i = 0; i < ndat2; i++)
        {
	    new_x = i*t_samp_new;
            old_low = (int)Math.floor(new_x/t_samp_old);
            old_x = old_low*t_samp_old;
            if(old_low < nfft-1)
            {
		old_y_low = tr[old_low];
                old_y_high = tr[old_low+1];
                /* linear interpolation */
                x1 = old_y_low + (old_y_high - old_y_low)*(new_x - old_x)/t_samp_old;
                k = (int)(((float)i)/(t_samp_old/t_samp_new));
		t[k] = x1;
            }
	}
	copyTraceData(t);
    }
    
    //
    // ---------- reconstruction for discretized signals ----------
    //
     public void reconstruction (double freq)
    {     
      double t, argu, delta_t;
      int nsamp, ndat, ii, nfft_cal;
      double t_samp_old, t_samp_new;
      double x1;
      int i, k;
      int ndat2;

	t_samp_old = t_samp;
        t_samp_new = 1.0/freq;
        x1 = t_samp_old/t_samp_new;
        x1 *= nfft;
        ndat2 = (int)Math.ceil(x1);

        /* calculate next power of 2 */
        ndat = 1;
        while(ndat < nfft)
	    ndat *= 2;
	nfft_cal = ndat;
	double t_new[] = new double[nfft_cal];
       	x1 = t_samp/t_samp_new;
	x1 *= nfft_cal;
	nsamp = (int)Math.ceil(x1);
	    
	double samples[] = new double[nsamp];
	double times [] = new double [nsamp];

	/* how many samples exist ? */
	for (i = 0; i < nsamp; i++)
	{
	    k = (int)(((float)i)/(t_samp/t_samp_new));
	    samples[i] = tr[k];
	    times[i] = t_samp * k;
	}
	
	/* perform Whittaker reconstruction */
	for (i = 0; i < nfft_cal; i++)
	{
	    t = i * t_samp;
	    argu = Math.sin(Math.PI*t/t_samp_new)/(Math.PI/t_samp_new);
	    tr[i] = 0.0;
	    for (ii = 0; ii < nsamp; ii++)
	    {
		delta_t = t_samp*((double)i - (double)ii * (t_samp_new/t_samp));
		k = (int)(((float)ii)*t_samp_new/t_samp);
		if((t - times[ii]) != 0)
		t_new[i] += (float)(((double)pow_I(-1,ii))*(samples[ii])/
                    delta_t);
	    }
	    t_new[i] *= argu;
	}
	for (i = 0; i < nsamp; i++)
	{
	    k = (int)(((float)i)/(t_samp/t_samp_new));
	    t_new[k] = samples[i];
	}
	copyTraceData (t_new);
    }  
 
    
    //
    // ---------- ADC Converter Simulation ----------
    //
  public void plainADC (double t_samp_new, double gainVal, int res,
			  int noGains, double lsb)
    {
      double t_samp_old;
      double x1;
      int i, k;
      int ndat2;
      int closest_sample = 0;
      int int_start_sample = 0;
      int j, jj;
      double preamp_gain, gain_up, out, y, lsb_val, err, new_x;
      int res_bits, keep_bits, sign;
      long no_gains;
    
	t_samp_old = t_samp;
	t_samp = t_samp_new;
		        	       
	/* reset ADC */
        gain = 0;
        y = 0.0;
        res_bits = 12;
        lsb_val = 0.1;
        gain_up = 1.0;
        no_gains = 1;
        out = quantize(y,res_bits,lsb_val,gain_up,no_gains);
        /* end reset ADC */

        preamp_gain = gainVal;
	    
        res_bits = res;
        /* how many bits to actually keep */
	keep_bits = res_bits;
        /* take one bit away for sign bit */
        res_bits -= 1;
        keep_bits -= 1;

        lsb_val = lsb;
        no_gains = pow_I (2,noGains);
	                
        /* gain switch fraction */
        gain_up = .25;

        x1 = t_samp_old/t_samp_new;
        x1 *= nfft;
        int nfft_new = (int)Math.ceil(x1);
	double tr_new[] = new double[nfft_new];
	
        for (j = 0; j < nfft_new;j++)
        {
            new_x = j*t_samp_new;
            /* new interpolation -> spline interpolation */

            /* determine index of closest sample in trace */
            closest_sample = (int)(new_x/t_samp_old);

            if ((closest_sample >= n_int/2) && (closest_sample <= nfft - n_int/2 - 1)) {
               int_start_sample = closest_sample - n_int/2 - 1;
            }
            else if (closest_sample < n_int/2) {
               int_start_sample = -1;
            }
            else if (closest_sample > nfft - n_int/2 - 1) {
               int_start_sample = nfft -1 - n_int;
            }
            for (jj = 1;jj <= n_int; jj++) {
               ya[jj] = tr[int_start_sample + jj];
               xa[jj] =  (int_start_sample + jj)*t_samp_old;
            }
	    x1 = polint(new_x);

            x1 *= preamp_gain;
            sign = 1;
            y = x1;
            if (x1 < 0) {
                 sign = -1;
                 y = -x1;
            }

            out = quantize(y,res_bits,lsb_val,gain_up,no_gains);

            while(out > (int)pow_I(2,keep_bits)-1) {
                out /= 2.0;
                gain +=1;
            }

            err = sign*out*pow_I(2,gain) - x1;  /*error signal */
	   // x1 = (param == PLAINADCQUANTIZED) ? sign*out*pow_I(2,gain) :
		// ((param == PLAINADCMANTISSA) ? sign*out :
		// (((param == PLAINADCEXPONENT) ? gain : err )));
	    x1 = (theLabel.equals(
		  plainADCSt+": "+plainADCQuantizedSt.substring(0,plainADCQuantizedSt.lastIndexOf("(")))) ? sign*out*pow_I(2,gain) :
		((theLabel.equals(plainADCSt+": "+plainADCMantissaSt)) ? sign*out :
		(((theLabel.equals(
		  plainADCSt+": "+plainADCExponentSt.substring(0,plainADCExponentSt.lastIndexOf("(")))) ? gain : err )));
            tr_new[j] = x1;
       }
       nfft = tr_new.length;
       copyTraceData (tr_new);
  } 
   
  //
  // ---------- FIR2CAUS ----------
  //    
  double fdig, time_tag, a[], b[];  
  int no_ar, no_ma;  
  
  public void fir2caus (String file,double freq,int pad_zeros, boolean correct_time)
  {	     /* pad_zeros = no. of padded zeroes for filtering  */
      int lead_zeros; 
      int ndat2, nskip = 0;; 
      double x[], x2[], y[], temp[];
      double shift_samples;
      int i,j,l,k, mx;
      
	proceeding_done = true;
	discr = false;
	if (!wg.ServerConnection.readFIRFile(file)) return;
	fdig = wg.ServerConnection.fdig;
	time_tag = wg.ServerConnection.time_tag;
	a = wg.ServerConnection.a;
	b = wg.ServerConnection.b;
	no_ar = wg.ServerConnection.no_ar;
	no_ma = wg.ServerConnection.no_ma;
	//param = 41;
	shift_samples = -time_tag;
	t_samp = (freq > 0) ? 1/freq : 1/fdig;
	lead_zeros = 0;
	if(pad_zeros > 0) lead_zeros = pad_zeros;
	ndat2 = wg.frame.graph.t.nfft - nskip + lead_zeros + pad_zeros;
	x = new double [ndat2+1];
	x2 = new double [ndat2+1];
	for(k = 0 ;k < ndat2-nskip-pad_zeros-lead_zeros; k++)
	    if(k >= nskip)
		x2[k-nskip+lead_zeros] = wg.frame.graph.t.tr[k];
	
	/* flip in time */
	for(k = 0; k < ndat2; k++)
	    x[k] = x2[ndat2-1-k];

	y = new double[ndat2+1];
		
	mx = no_ar;

	// filter 
	for (i = 0; i < ndat2; i++)          
	{
	    y[i] = 0.0;
	    /* MA */
	    for (l = 0; l <= mx; l++)
		if ((i-l) >= 0) 
		    y[i] += x[i-l]*b[l];
            /* AR */
	    for (k = 1; k <= mx; k++) 
		if ((i-k) >= 0) 
                y[i] += y[i-k]*a[k];
        }
	
	// flip back in time
	temp = new double [ndat2+1];
	for(k = 0; k < ndat2; k++)
	    temp[k]= y[ndat2-1-k];
	    
	if (correct_time)
   	    time_shift(temp,ndat2,-shift_samples);
	    
	if(pad_zeros == 0)
	{
	    tr = new double [ndat2-lead_zeros];
	    nfft = ndat2-lead_zeros;
	    /* write out trace without padding zeros */
	    for(j = 0; j < ndat2-lead_zeros; j++)
               tr[j] = temp[j+lead_zeros];
	}       
	else
	{
	    tr = new double [ndat2];
	    nfft = ndat2;
	    /* write out trace with padding zeros */
	    for(j = 0; j < ndat2; j++)
                tr[j] = temp[j];
        }
    }
  
    public void time_shift(double y[],int ndat, double shift_samples)
    {
      double amp,phase;		// amplitude and phase of shifting spectrum 
      double real, imag;        // real, imaginary of shifting spectrum 
      double t_samp;            // sampling interval 
      int ia;                   // integer part of shift 
      double extra_samples;     // fraction of samples to shift 
      double b1[];              // trace buffers 
      double bb1[], bb2[];      // trace buffers 
      double x1;
      int nf;
      int i,j;
      
	/*  allocate buffer */
	b1 = new double[ndat];
	ia = 0;
	if(shift_samples == (int)shift_samples)
	{ /*integer multiple of 1 sample */
	    ia = (int)shift_samples;
	    extra_samples = 0.0;
	}
	else
	{
	    ia = (int)shift_samples;
	    extra_samples = shift_samples - ia;
	}
	if(ia >0)
	{
	    for (j = ndat-1 ; j >= ia; j--)
	    {
		x1 = y[j - ia];
		b1[j] = x1;
	    }
	    for (j = 0; j < ia; j++)
		b1[j] = 0.0;
	}
	else if(ia < 0)
	{
	    for (j = 0 ; j < ndat +ia -1 ; j++)
	    {
		x1 = y[j - ia];
		b1[j] = x1;
	    }
	    for (j = ndat-ia ; j <ndat; j++)
	    {
		b1[j] = 0.0;
	    }
	}
	else if (ia == 0)
	{
	    for (j = 0; j < ndat; j++)
		b1[j] = y[j];
	}
	/*
	every shift less than a sample is done in the
	frequency domain
	 */
	if (extra_samples != 0.0) /* non-integer part of shift */
	{
	    nf = 1;
	    while(nf < ndat)
		nf *= 2;
	    nf *= 2; /* to avoid wrap around */
	    nfft = nf;
	    bb1 = new double [nf];
	    bb2 = new double [nf];
	    for(j = 0; j < ndat;j++)
	    {
		bb1[j] = b1[j];
	    }
	    realft(bb1,nf,1,nf/2);
	    amp = 1.0;
	    /*
	    1 sample shift == phase of PI at f Nyquist (linear in between)
	    */
	    bb2[0] = amp;
	    bb2[1] = amp;
	    for (j = 1; j < nf/2; j++)
	    {
		phase = extra_samples*((double)j/(double)nf)*Math.PI;
		real = amp*Math.cos(phase);
		imag = amp*Math.sin(phase);
		bb2[2*j] = real;
		bb2[2*j+1] = imag;
	    }
	    /* spectral multiplication */
	    bb1[0] *= bb2[0];
	    bb1[1] *= bb2[1];
	    for (j = 1; j < nf/2; j++)
	    {
		real = bb1[2*j]*bb2[2*j] - bb1[2*j+1]*bb2[2*j+1];
		imag = bb1[2*j]*bb2[2*j+1] + bb1[2*j+1]*bb2[2*j];
		bb1[2*j] = real;
		bb1[2*j+1] = imag;
	    }
	    /* inverse FFT */
	    realft(bb1,nf,-1,nf/2);
	    /* scale amplitudes back */
	    for(j = 0; j < ndat; j++)
	    {
		y[j] = 2*bb1[j]/nf;
	    }
       
	}
	else
	{
	    for(j = 0; j < ndat; j++)
	    {
		y[j] = b1[j];
	    }
	}    
   }
      
//    
// ---------- Interpolate ---------
//
    
// E. Wielandt's filter coefficients for interpolation ratios 1 - 6 
// Only the ratios 2, 3, and 5 are used presently.E. g. an interpolation 
// by a factor of 4  is performed as 2 x 2 interpolations 

/* g2_1 RMS rel. error for k_rel <0.8: 0.009% */
static double g2_1[] = 
{ -0.0002616,  0.0009302, -0.0023258,  0.0049142, -0.0092537,  0.0161355, -0.0266150,  0.0424554, 
  -0.0670612,  0.1091686, -0.2008408,  0.6327541,  0.6327542, -0.2008408,  0.1091686, -0.0670612, 
   0.0424554, -0.0266150,  0.0161355, -0.0092537,  0.0049142, -0.0023258,  0.0009302, -0.0002616};

/* g3_1 RMS rel. error for k_rel <0.8: 0.008% */
static double g3_1[] = 
{ -0.0002324,  0.0008256, -0.0020654,  0.0043684, -0.0082410,  0.0144087, -0.0238643,  0.0383080,
  -0.0611438,  0.1015046, -0.1959616,  0.8226883,  0.4111037, -0.1564917,  0.0885536, -0.0553522,
   0.0353777, -0.0223077,  0.0135759, -0.0078056,  0.0041522, -0.0019670,  0.0007871, -0.0002211};

/* g4_1 RMS rel. error for k_rel <0.8: 0.007% */
static double g4_1[] = 
{ -0.0001922,  0.0006827, -0.0017085,  0.0036157, -0.0068275, 0.0119544, -0.0198421,  0.0319579, 
  -0.0512943,  0.0860733, -0.1708677,  0.8964090,  0.2985412, -0.1217245, 0.0701113, -0.0441759,
   0.0283604, -0.0179323,  0.0109329, -0.0062935,  0.0033505, -0.0015879, 0.0006355, -0.0001784};

/* g5_1 RMS rel. error for k_rel <0.8: 0.006% */
static double g5_1[] = 
{ -0.0001611,  0.0005720, -0.0014316,  0.0030307, -0.0057263,  0.0100352, -0.0166788,  0.0269186,
  -0.0433567,  0.0732492, -0.1480766,  0.9320452,  0.2327664, -0.0984035,  0.0572469, -0.0362360,
   0.0233232, -0.0147709,  0.0090151, -0.0051932,  0.0027660, -0.0013112,  0.0005248, -0.0001473};

/* g5_2 RMS rel. error for k_rel <0.8: 0.009% */
static double g5_2[] =
{ -0.0002526,  0.0008977, -0.0022452,  0.0047467, -0.0089479,  0.0156272, -0.0258392,  0.0413715,
  -0.0657512,  0.1082703, -0.2048060,  0.7525200,  0.5015040, -0.1790148,  0.0997642, -0.0619420,
   0.0394431, -0.0248145,  0.0150789, -0.0086611,  0.0046043, -0.0021804,  0.0008723, -0.0002452};

/* g6_1 RMS rel. error for k_rel <0.8: 0.005% */
static double g6_1[] =
{ -0.0001378,  0.0004891, -0.0012243,  0.0025926, -0.0049006,  0.0085933, -0.0142952,  0.0231041,
  -0.0373015,  0.0633145, -0.1296402,  0.9518885,  0.1901555, -0.0822067,  0.0481311, -0.0305567,
   0.0197009, -0.0124900,  0.0076282, -0.0043963,  0.0023423, -0.0011105,  0.0004445, -0.0001247};

static int g2_1_len=24;
static int g3_1_len=24;
static int g4_1_len=24;
static int g5_1_len=24;
static int g5_2_len=24;
static int g6_1_len=24;   

 public void Interpolate(int ipol_fac, int no_it2, int no_it3, int no_it5)
  {
    int k;
    double tr_out[], tr_in[];
	t_samp /= ipol_fac;
	tr_in = new double [nfft];
	tr_in = tr;
	
	/* interpolations by factors of  2 */
	for(k = 0; k < no_it2; k++) {
	    tr_out = new double [nfft*2];      
	    ipol2(tr_in,tr_out,nfft);
	    nfft *= 2;
            tr_in = tr_out;
	}
	/* interpolations by factors of 3 */
	for(k = 0; k < no_it3; k++) {
	    tr_out = new double [nfft*3];      
	    ipol3(tr_in,tr_out,nfft);
	    nfft *= 3;
            tr_in = tr_out;
	}
	/* interpolations by factors of  5 */
	for(k = 0; k < no_it5; k++) {
	    tr_out = new double [nfft*5];      
	    ipol5(tr_in,tr_out,nfft);
	    nfft *= 5;
            tr_in = tr_out;
	}
	tr = tr_in;
  }
  
   /* interpolation by factor 2 */
  public void  ipol2(double [] xin, double [] xout,int n)
  {
    double xx;
    int i,j, k;
    /* xx = interpolated sample at center between index i and i+1 */
    for(i = 0; i < n-1; i++)
    {
        xx =0.0;
        for(j = 0; j < g2_1_len/2; j++)
        {
            if((i-j) >= 0) /* left side of filter */
               xx+=g2_1[g2_1_len/2-(j+1)]*xin[i-j];
            if((i+1+j) < n) /* right side of filter */
               xx+=g2_1[g2_1_len/2+j]*xin[i+1+j];
        }
        xout[2*i] = xin[i];
        xout[2*i+1] = xx;
        xout[2*i+2] = xin[i+1];
    }
  }
 
  /* interpolation by factor 3 */
  public void  ipol3(double [] xin, double [] xout,int n)
  {
    double xx;
    int i,j, k;
    int ifac = 3; /* interpolation factor */
    
    for(i = 0; i < n-1; i++)
    {
        xout[ifac*i] = xin[i];
        xout[ifac*i+ifac] = xin[i+1];
    }

    /* here xx = interpolated sample 1/3 between index i and i+1 */
    for(i = 0; i < n-1; i++)
    {
        xx =0.0;
        for(j = 0; j < g3_1_len/2; j++)
        {
            if((i-j) >= 0) /* left side of filter */
               xx+=g3_1[g3_1_len/2-(j+1)]*xin[i-j];
            if((i+1+j) < n) /* right side of filter */
               xx+=g3_1[g3_1_len/2+j]*xin[i+1+j];
        }
        xout[ifac*i+1] = xx;
    } 

    /* here xx = interpolated sample 2/3 between index i and i+1 */
    /*       use g3_1[] in reverse */ 
    for(i = 0; i < n-1; i++)
    {
        xx =0.0;
        for(j = 0; j < g3_1_len/2; j++)
        {
            if((i-j) >= 0) /* left side of filter */
               xx+=g3_1[g3_1_len/2+j]*xin[i-j];
            if((i+1+j) < n) /* right side of filter */
               xx+=g3_1[g3_1_len/2-(j+1)]*xin[i+1+j];
        }
        xout[ifac*i+2] = xx;
     }
  }

  /* interpolation by factor 5 */
  void  ipol5(double [] xin, double [] xout,int n)
  {
    double xx;
    int i,j, k;
    int ifac = 5; /* interpolation factor */
    
    for(i = 0; i < n-1; i++)
    {
        xout[ifac*i] = xin[i];
        xout[ifac*i+ifac] = xin[i+1];
    }

    /* here xx = interpolated sample 1/5 between index i and i+1 */
    for(i = 0; i < n-1; i++)
    {
        xx =0.0;
        for(j = 0; j < g5_1_len/2; j++)
        {
            if((i-j) >= 0) /* left side of filter */
               xx+=g5_1[g5_1_len/2-(j+1)]*xin[i-j];
            if((i+1+j) < n) /* right side of filter */
               xx+=g5_1[g5_1_len/2+j]*xin[i+1+j];
        }
        xout[ifac*i+1] = xx;
    } 

    /* here xx = interpolated sample 2/5 between index i and i+1 */
    for(i = 0; i < n-1; i++)
    {
        xx =0.0;
        for(j = 0; j < g5_2_len/2; j++)
        {
            if((i-j) >= 0) /* left side of filter */
               xx+=g5_2[g5_2_len/2-(j+1)]*xin[i-j];
            if((i+1+j) < n) /* right side of filter */
               xx+=g5_2[g5_2_len/2+j]*xin[i+1+j];
        }
        xout[ifac*i+2] = xx;
    } 

    /* here xx = interpolated sample 3/5 between index i and i+1 */
    /*       use g5_2[] in reverse */ 
    for(i = 0; i < n-1; i++)
    {
        xx =0.0;
        for(j = 0; j < g5_2_len/2; j++)
        {
            if((i-j) >= 0) /* left side of filter */
               xx+=g5_2[g5_2_len/2+j]*xin[i-j];
            if((i+1+j) < n) /* right side of filter */
               xx+=g5_2[g5_2_len/2-(j+1)]*xin[i+1+j];
        }
        xout[ifac*i+3] = xx;
     }

    /* here xx = interpolated sample 4/5 between index i and i+1 */
    /*       use g5_1[] in reverse */ 
    for(i = 0; i < n-1; i++)
    {
        xx =0.0;
        for(j = 0; j < g5_1_len/2; j++)
        {
            if((i-j) >= 0) /* left side of filter */
               xx+=g5_1[g5_1_len/2+j]*xin[i-j];
            if((i+1+j) < n) /* right side of filter */
               xx+=g5_1[g5_1_len/2-(j+1)]*xin[i+1+j];
        }
        xout[ifac*i+4] = xx;
     }
   }  
 
// --------------- Butterworth Band Pass ---------------------------------------
  
 public void butterworthBandPass (double flo, double fhi, int ns, String zph)
 {
    int k;                   
    int n, m, mm;
    double a[] = new double [butterworthMAXSEC+1];
    double b[] = new double [butterworthMAXSEC+1];
    double c[] = new double [butterworthMAXSEC+1];
    double d[] = new double [butterworthMAXSEC+1];
    double e[] = new double [butterworthMAXSEC+1];
    double f[][] = new double [butterworthMAXSEC+1][6];

    double temp;
    double c1,c2,c3;
    double w1,w2,wc,q,p,r,s,cs,x;

    /* design filter weights */
    /* bandpass */
    w1 = Math.sin (flo * Math.PI * t_samp) / Math.cos (flo * Math.PI * t_samp);
    w2 = Math.sin (fhi * Math.PI * t_samp) / Math.cos (fhi * Math.PI * t_samp);
    wc = w2 - w1;
    q = wc * wc + 2.0 * w1 * w2;
    s = w1 * w1 * w2 * w2;
    for (k = 1; k <= ns; k++) {
	c1 = (double)(k + ns);
        c2 = (double)(4 * ns);
        c3 = (2.0 * c1 - 1.0) * Math.PI / c2;
        cs = Math.cos(c3);
        p = -2.0 * wc * cs;
        r = p * w1 * w2;
        x = 1.0 + p + q + r + s;
        a[k] = wc * wc / x;
        b[k] = (-4.0 -2.0*p+ 2.0*r+4.0*s) / x;
        c[k] = (6.0 - 2.0*q +6.0*s) / x;
        d[k] = (-4.0 +2.0*p -2.0*r +4.0*s) / x;
        e[k] = (1.0 - p + q - r +s) / x;
    }
/* filtering */
    for (m = 1; m <= nfft; m++) {
            f[1][5]= tr[m-1];
            /* go thru ns filter sections */
            for (n = 1; n <= ns; n++) {
                    temp = a[n]*(f[n][5]-2.0*f[n][3] +f[n][1]);
                    temp = temp-b[n]*f[n+1][4]-c[n]*f[n+1][3];
                    f[n+1][5] = temp-d[n]*f[n+1][2]-e[n]*f[n+1][1];
            }
            /* update past values */
            for(n = 1; n <= ns + 1; n++)
                    for(mm = 1; mm <= 4; mm++)
                            f[n][mm]=f[n][mm+1];
            /* set present data value and continue */
            tr[m-1] = f[ns+1][5];
    }
    if (zph.equals (butterworthBackwardSt)) {
        /* filtering reverse signal*/
        for (m = nfft; m >= 1; m--) {
                f[1][5]= tr[m-1];
                /* go thru ns filter sections */
                for(n = 1; n <= ns; n++) {
                        temp = a[n]*(f[n][5]-2.0*f[n][3] +f[n][1]);
                        temp = temp-b[n]*f[n+1][4]-c[n]*f[n+1][3];
                        f[n+1][5] = temp-d[n]*f[n+1][2]-e[n]*f[n+1][1];
                }
                /* update past values */
                for(n = 1; n <= ns+1; n++)
                        for(mm=1;mm<=4;mm++)
                                f[n][mm]=f[n][mm+1];
                /* set present data value and continue */
                tr[m-1] = f[ns+1][5];
        }
    }
 }
   
// --------------- Butterworth Low Pass ---------------------------------------
  
 public void butterworthLowPass (double fc, int ns, String zph)
 {
    int k;                      
    int n,m,mm;
    double a[] = new double [butterworthMAXSEC+1];
    double b[] = new double [butterworthMAXSEC+1];
    double c[] = new double [butterworthMAXSEC+1];
    double f[][] = new double [butterworthMAXSEC+1][6];
 
    double temp;
    double wcp,cs,x;
 
    /* design filter weights */
    wcp = Math.sin(fc*Math.PI*t_samp)/Math.cos(fc*Math.PI*t_samp);
    for (k=1;k<=ns;k++) {
            cs=Math.cos((2.0*(k+ns)-1.0)*Math.PI/(4.0*ns));
            x=1.0/(1.0+wcp*wcp -2.0*wcp*cs);
            a[k]=wcp*wcp*x;
            b[k]=2.0*(wcp*wcp-1.0)*x;
            c[k]=(1.0 +wcp*wcp +2.0*wcp*cs)*x;
    }
    /* set initial values to 0 */
    for(n=0;n<=butterworthMAXSEC;n++)
            for(m=0;m<=5;m++)
                    f[n][m]=0.0;
  /* set initial values to 0 */
    /* filtering */
    for (m=1;m<=nfft;m++)
    {
            f[1][3]= tr[m-1];
            /* go thru ns filter sections */
            for(n=1;n<=ns;n++) {
                    temp=a[n]*(f[n][3]+2.0*f[n][2] +f[n][1]);
                    f[n+1][3]=temp-b[n]*f[n+1][2]-c[n]*f[n+1][1];
            }
            /* update past values */
            for(n=1;n<=ns+1;n++)
                    for(mm=1;mm<=2;mm++)
                            f[n][mm]=f[n][mm+1];
            /* set present data value and continue */
            tr[m-1]=f[ns+1][3];
    }
    if (zph.equals (butterworthBackwardSt))
    {
        /* filtering reverse signal*/
        for (m=nfft;m>=1;m--)
        {
            f[1][3]= tr[m-1];
            /* go thru ns filter sections */
            for(n=1;n<=ns;n++) {
                    temp=a[n]*(f[n][3]+2.0*f[n][2] +f[n][1]);
                    f[n+1][3]=temp-b[n]*f[n+1][2]-c[n]*f[n+1][1];
            }
            /* update past values */
            for(n=1;n<=ns+1;n++)
                    for(mm=1;mm<=2;mm++)
                            f[n][mm]=f[n][mm+1];
            /* set present data value and continue */
            tr[m-1]=f[ns+1][3];
        }
    }

}
   
// --------------- Butterworth High Pass ---------------------------------------
  
 public void butterworthHighPass (double fc, int ns, String zph)
 {
    int k;                      
    int n,m,mm;
    double a[] = new double [butterworthMAXSEC+1];
    double b[] = new double [butterworthMAXSEC+1];
    double c[] = new double [butterworthMAXSEC+1];
    double f[][] = new double [butterworthMAXSEC+1][6];
 
    double temp;
    double wcp,cs;
    /* design filter weights */
    wcp=Math.sin(fc*Math.PI*t_samp)/Math.cos(fc*Math.PI*t_samp);
    for (k=1;k<=ns;k++)
    {
            cs=Math.cos((2.0*(k+ns)-1.0)*Math.PI/(4.0*ns));
            a[k]=1.0/(1.0+wcp*wcp-2.0*wcp*cs);
            b[k]=2.0*(wcp*wcp-1.0)*a[k];
            c[k]=(1.0 +wcp*wcp +2.0*wcp*cs)*a[k];
    }

    /* set initial values to 0 */
    for(n=0;n<=butterworthMAXSEC;n++)
            for(m=0;m<=5;m++)
                    f[n][m]=0.0;
    /* set initial values to 0 */
    /* filtering */
    for (m=1;m<=nfft;m++)
    {
            f[1][3]= tr[m-1];
            /* go thru ns filter sections */
            for(n=1;n<=ns;n++)
            {
                    temp=a[n]*(f[n][3]-2.0*f[n][2] +f[n][1]);
                    f[n+1][3]=temp-b[n]*f[n+1][2]-c[n]*f[n+1][1];
            }
            /* update past values */
            for(n=1;n<=ns+1;n++)
                    for(mm=1;mm<=2;mm++)
                            f[n][mm]=f[n][mm+1];
            /* set present data value and continue */
            tr[m-1]=f[ns+1][3];
    }
    if (zph.equals (butterworthBackwardSt))
    {
        /* filtering reverse signal*/
        for (m=nfft;m>=1;m--)
        {
            f[1][3]= tr[m-1];
            /* go thru ns filter sections */
            for(n=1;n<=ns;n++)
            {
                    temp=a[n]*(f[n][3]-2.0*f[n][2] +f[n][1]);
                    f[n+1][3]=temp-b[n]*f[n+1][2]-c[n]*f[n+1][1];
            }
            /* update past values */
            for(n=1;n<=ns+1;n++)
                    for(mm=1;mm<=2;mm++)
                            f[n][mm]=f[n][mm+1];
            /* set present data value and continue */
            tr[m-1]=f[ns+1][3];
        }
    }

   }
  
 // 
 //------------ Delat Modulation ---------------
 //
    public void deltaModulation(double LSB)
    {
	copyTraceData (dmod (tr, nfft,LSB,t_samp));
    }
	
    public double [] dmod(double tr1[], int ndat, double lsb, double t_samp)
    {
      double xq[];
      double quin[];
      double y[];
      int j;
	
	xq  = new double [ndat+1];	// input stream reconstructed from quantized values
	quin = new double [ndat+1];	// input stream for the single bit quantizer 
	y = new double [ndat];		// quantizer output stream 
	
	/* first output sample of quantizer */     
	if (tr1[0] >= 0)
	    y[0] = lsb/t_samp;
	else
	    y[0] = -lsb/t_samp;
	 /* integrate first sample */ 
	xq[0] += y[0]*t_samp/2;

	for (j = 1; j < ndat;j++)
	{
	    quin[j] = tr1[j] - xq[j-1];
	    if(quin[j] >= 0)
		y[j] = lsb/t_samp;
	    else
		y[j] = -lsb/t_samp; 
	    xq[j] =  xq[j-1] + y[j]*t_samp;
	}
	
	return (y);
     }
     
 // 
 //------------ Sigma Delat Modulation ---------------
 //
    public void sigmaDeltaModulation (double LSB)
    {
	discr = true;
	copyTraceData(sdmod (LSB));
    }
    
    public double [] sdmod(double lsb)
    {
      int j;
      double sum[], quin[], y[];
     
	sum  = new double [nfft+1];	// sum of quantized stream and input stream
	quin  = new double [nfft+1];	// integrated sum stream = input stream for the single bit quantizer
        y  =  new double [nfft];

	if (tr[0] >= 0)
	    y[0] = lsb;
	else
	    y[0] = -lsb;
	quin[0] = y[0]*t_samp/2.;

	for (j = 1; j < nfft;j++)
	{
	    sum[j] = tr[j] - y[j-1];
	    quin[j] = quin[j-1] + sum[j]*t_samp;
	    if(quin[j] >= 0)
		y[j] = lsb;
	    else
		y[j] = -lsb;
	}
	return (y);
    }
}     