//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.event.*;
import java.awt.*;

class ResponseCommand extends Command
{
 int axisType;
 String responseType;
 String element;
 int points;
 
    public ResponseCommand (WinGroup wg, String what)
    {
	super (wg);
	responseType = what;
	if (what.equals (stepResponseSt)) id = STEPRESPONSE;
	if (what.equals (impulseResponseSt)) id = IMPULSERESPONSE;
    }
    
    public ResponseCommand (WinGroup wg, int idif)
    {
	super (idif, wg); 
	if (idif == STEPRESPONSE) responseType = stepResponseSt;
	if (idif == IMPULSERESPONSE) responseType = impulseResponseSt;
    }
    
    public ResponseCommand (WinGroup wg, String parent, String what, int axis)
    {
	super (wg); 
    	axisType = axis;
	element = what;
	responseType = parent;
	if (parent.equals (frequencyResponseSt)) id = FREQUENCYRESPONSE;
	else id = SPECTRUM;
	findId (what);
    }
    
    public ResponseCommand (WinGroup wg, String what, int axis)
    {
	super (FREQUENCYRESPONSE, wg); 
	axisType = axis;
	element = what;
	responseType = frequencyResponseSt;
	id = FREQUENCYRESPONSE;
	findId (what);
    }
    
    protected void findId (String what)
    {
    	if (what.equals (specAmplitudeSt)) id += AMPL;
	if (what.equals (specPhaseSt)) id += PHAS;
	if (what.equals (specRealSt)) id += REAL;
	if (what.equals (specImaginarySt)) id += IMAG;
	if (what.equals (specGroupDelaySt)) id += GROU;
	if (what.equals (specPhaseDelaySt)) id += PHDE;
    }
    
    public void actionPerformed (ActionEvent e)
    { 
	String s = e.getActionCommand();
	if (s.equals (responseSt)) {
	    if (SPECTRUM+AMPL <= id && id <= SPECTRUM+IMAG) {
		if(wg.frame.graph.t.getId() == -1) {
		    wg.message.setVisible ("No data to proceed !");
		    return;
		}
	    } 
	    else if(!wg.CalVal.haveCalibration()) {
		wg.message.setVisible ("No calibration given");
		return;
	    }
	    switch (id) {
		case IMPULSERESPONSE:
		    if (wg.params.getSetupPoints() < wg.params.getSetupSpikePosition()) {
		        wg.message.setVisible
		    	("Number of spike position is too big ! Modify Setup first !");
		        return;
		    }
		    break;
		case STEPRESPONSE:
		    if (wg.params.getSetupPoints() < wg.params.getSetupStepPosition()) {
			wg.message.setVisible
			("Number of step position is too big ! Modify Setup first !");
			return;
		    }
		    break;
	    }
	    if ( !wg.checkPoints(wg.params.getSetupPoints()) ) {
	    	wg.settings.newSettings (id);
	    	wg.editor.newSettings (wg.settings, this);
		wg.editor.showIt();
		return;
	    }
	    else {
		points = wg.params.find_next_nfft();
		actionPerformed(new ActionEvent (wg, Event.ACTION_EVENT, "startResponse"));
	    }
	}
	
	if (s.equals ("startResponse")) {
	    Graph myGraph = wg.getGraph(id);
	    switch (id) {
		case FREQUENCYRESPONSE+AMPL:
		case FREQUENCYRESPONSE+PHAS:
		case FREQUENCYRESPONSE+REAL:
		case FREQUENCYRESPONSE+IMAG:
		case FREQUENCYRESPONSE+GROU:
		case FREQUENCYRESPONSE+PHDE:
		    wg.specGSE.selectResponse (element, axisType);
		    myGraph.SetPlotTrace(wg.specGSE);
		    if (wg.FitResponseCalVal.haveCalibration())
		    {
			wg.FitResponse();
			wg.fitSpecTrace.selectResponse (element, axisType);
			myGraph.SetSecondPlotTrace (wg.fitSpecTrace, false);
		    }
		    break;
		case IMPULSERESPONSE:
		case STEPRESPONSE:
		    TestSignal tS = new TestSignal (wg, id == IMPULSERESPONSE ? IMPULSE: STEP);
		    tS.buildSignal();
		    Spectrum spectrum = new Spectrum
		    (points, 1/wg.params.getSetupFrequency(), wg.CalVal);
		    ProceedSignal pS = new ProceedSignal (wg, tS, id);
		    pS.BuildGSEResponseTrace (points, spectrum.tr);
		    myGraph.SetPlotTrace (pS);
		    if (wg.FitResponseCalVal.haveCalibration())
		    {
			Spectrum fitResponseSpectrum = new Spectrum
			    (points, 1/wg.params.getSetupFrequency(), wg.FitResponseCalVal);
			ProceedSignal fitPS = new ProceedSignal (wg, tS, id);
			fitPS.BuildGSEResponseTrace (points, fitResponseSpectrum.tr);
			myGraph.SetSecondPlotTrace (fitPS, false);
		    }
		    break;
		case SPECTRUM+AMPL: 
		case SPECTRUM+PHAS:
		case SPECTRUM+REAL:
		case SPECTRUM+IMAG:
		    wg.specTrace = new TraceSpectrum (new PlotTrace (wg.frame.graph.t, points/2));
		    wg.specTrace.enablePainting();
		    wg.specTrace.setAsSpectrum (element, axisType);
		    myGraph.SetPlotTrace (wg.specTrace);
		    break;
	    }
	    if (!(myGraph.getParent()).isVisible()) (myGraph.getParent()).setVisible(true);
	    myGraph.repaint();
	}	
	
    // arrive here after input of points
	if (s.equals (cancelSt)) {
	    wg.editor.setVisible(false);
	    return;
	}
	if (s.equals (okSt) || s.equals (applySt)) {
	    points = wg.editor.getInteger (numberOfPointsForFFTSt);
	    if (points != Integer.MIN_VALUE) {
	        if (!wg.checkPoints (points)) {	
		    wg.message.setVisible ("Number of Points must be a power of 2 !");
		    return;	
		}
		if (s.equals (okSt)) wg.editor.setVisible(false);
		actionPerformed(new ActionEvent (wg, Event.ACTION_EVENT, "startResponse"));
		return;
	    }    
	}	
    }

 }