//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.event.*;
import java.awt.*;
import java.util.*;

class TraceCommand extends Command
{
    public TraceCommand (int id, WinGroup wg){ super (id, wg); }
    
    public void actionPerformed (ActionEvent e) {
	String s = e.getActionCommand();
	if (s.equals (cancelSt)) {
	    wg.editor.setVisible(false);
	    return;
	}
	TestSignal signal = new TestSignal(wg, id);
	switch (id) {
	    case SINUS: {
		    double amplitude = wg.editor.getDouble (amplitudeSt);
		    double frequency = wg.editor.getDouble (frequencySt);
		    double phase = wg.editor.getDouble (phaseSt);
		    if (amplitude != Double.NEGATIVE_INFINITY && 
			frequency != Double.NEGATIVE_INFINITY &&
			phase != Double.NEGATIVE_INFINITY) {
			if (frequency < 0) {
			    wg.message.setVisible ("No valid frequency !");
			    return;
			}
			wg.params.saveParameter (this, amplitude, frequency, phase);
		    } else return;
		    break;
	    }
	    case RICKER:
	    case CONSTANT: {
		    double val = wg.editor.getDouble (id==CONSTANT ? amplitudeSt : frequencySt);
		    if (val != Double.NEGATIVE_INFINITY) {
			if (id == RICKER && val < 0) {
			    wg.message.setVisible ("No valid frequency !");
			    return;
			}
		        wg.params.saveParameter (this, val);
		    } else return;
		    break;
	    }
	    case MIXED:
	    case BRUNE: {
		    double val1 = wg.editor.getDouble (id==BRUNE ? bruneRthSt : amplitudeSt);
		    double val2 = wg.editor.getDouble (id==BRUNE ? bruneSigmaSt : frequencySt);
		    double val3 = wg.editor.getDouble (id==BRUNE ? bruneMuSt : phaseSt);
		    double val4 = wg.editor.getDouble (id==BRUNE ? bruneVsSt : mixedBSt);
		    double val5 = wg.editor.getDouble (id==BRUNE ? bruneRSt : mixedCSt);
		    double val6 = wg.editor.getDouble (id==BRUNE ? bruneZSt : mixedDSt);
		    if (val1 != Double.NEGATIVE_INFINITY && val2 != Double.NEGATIVE_INFINITY &&
			val3 != Double.NEGATIVE_INFINITY && val4 != Double.NEGATIVE_INFINITY &&
			val5 != Double.NEGATIVE_INFINITY && val6 != Double.NEGATIVE_INFINITY) {
			if (id == MIXED && val2 < 0) {
			    wg.message.setVisible ("No valid frequency !");
			    return;
			}
			wg.params.saveParameter (this, val1, val2, val3, val4, val5, val6);
		    } else return;
		    break;
	    }
	    case CHIRP: {
		    double amplitude = wg.editor.getDouble (amplitudeSt);
		    double frequency1 = wg.editor.getDouble (chirpFrequency1St);
		    double phase = wg.editor.getDouble (phaseSt);
		    double frequency2 = wg.editor.getDouble (chirpFrequency2St);
		    if (amplitude != Double.NEGATIVE_INFINITY && 
			frequency1 != Double.NEGATIVE_INFINITY &&
			frequency1 != Double.NEGATIVE_INFINITY &&
			phase != Double.NEGATIVE_INFINITY) {
			if (frequency1 < 0 || frequency2 < 0) {
			    wg.message.setVisible ("No valid frequency !");
			    return;
			}
			wg.params.saveParameter
			    (this, amplitude, frequency1, phase, frequency2, 0, 0);
		    } else return;
		    break;
	    }
	    case SEISMOMETER_DIDR:
	    case SEISMOMETER_ASDR:
	    case SEISMOMETER_ASVR: {
		    double frequency = wg.editor.getDouble (seismometerFrequencySt);
		    double damping = wg.editor.getDouble (seismometerDampingSt);
		    if (frequency != Double.NEGATIVE_INFINITY && 
			damping != Double.NEGATIVE_INFINITY) {
			if (frequency < 0) {
			    wg.message.setVisible ("No valid frequency !");
			    return;
			}	
			if (damping < 0 || damping >= 1.0) {
			    wg.message.setVisible ("No valid damping !");
			    return;
			}
			wg.params.saveParameter (this, frequency, damping);
		    } else return;
		    break;
	    }
	    case SPIKES: {
		    int anz;
		    int nfft = wg.params.getSetupPoints();
		    Vector index = wg.editor.getVector (spikesPositionSt);
		    Vector amplitude = wg.editor.getVector (amplitudeSt);
		    if (index != null && amplitude != null) {
			if (index.size() != amplitude.size() || (anz = index.size()) == 0) {
			    wg.message.setVisible ("No valid input for index and amplitude");
			    return;
			}	
			for (int i = 0; i < anz; i++)
			    if ((Integer.valueOf((String)index.elementAt(i))).intValue() >= nfft ||
				 (Integer.valueOf((String)index.elementAt(i))).intValue() < 0) {
				wg.message.setVisible 
				    ((Integer.valueOf((String)index.elementAt(i))).intValue()+
				    " is no valid point (0..."+nfft+")");
			    return;
			} 
			 wg.params.saveParameter (this, amplitude, index);
		    } else return;
		    break;
	    }
	    case DATATRACE: {
		    String dir = wg.editor.getSelectedItem( dataTraceFoldersSt);
		    String file = wg.editor.getSelectedItem (dataTraceFilesSt);
		    double freq = wg.editor.getDouble (dataTraceSamplingFrequencySt);
		    if (dir != null && file != null && freq != Double.NEGATIVE_INFINITY) {
			wg.params.saveParameter (this, freq);
			wg.params.saveParameter (this, dir+"/"+file);
		    } else return;
		    break;
	    }
	}
	signal.buildSignal ();
	if (s.equals (okSt)) wg.editor.setVisible(false);
	wg.frame.graph.SetPlotTrace(signal);
	wg.actionPerformed (new ActionEvent (this, Event.ACTION_EVENT, "graphRepaint")); 
    }
}