//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

public class TraceSpectrum extends PlotTrace{
 
  double tr_amplitude[], tr_phase[], tr_real[], tr_imag[], tr_groupd[], tr_phasedl[];
  double  t_amplitude[],  t_phase[],  t_real[],  t_imag[],  t_groupd[],  t_phasedl[];
   
  // mark, if painting is possible or not
  boolean amplitude,phase,real,imag,groupd,phasedl;
  int old_nfft;
  
  TraceSpectrum (PlotTrace t)
  {
    int i; 
    
    amplitude = phase = real = imag = groupd = phasedl = false;
    t_samp = t.t_samp;
    nfft = t.nfft;
    old_nfft = t.nfft;
	
  // ---- multiplex trace
    t_amplitude = new double [nfft];
    t_phase = new double [nfft];
    t_real = new double [nfft];
    t_imag = new double [nfft];
    t_groupd = new double [nfft];
    t_phasedl = new double [nfft];
    double real, imag, amp, phase;
    double angl, cor,dx;
    
    // Amplitude			
    for (i=1; i < nfft/2; i++)
    {
    	real = t.tr[2*i];
	imag = t.tr[2*i+1];
	amp = Math.sqrt(real*real +imag*imag);
	if(!((imag == 0.0) && (real == 0.0)))
	    phase = Math.atan2(imag,real);
	else
	    phase =0.0;
	t_amplitude[2*i] = amp;
	t_amplitude[2*i+1] = phase;
    }
    angl = Math.PI;
    cor = 0.0;
    for (i = 2; i < nfft/2; i++)
    {
    	phase = t_amplitude[2*i+1];
	dx = phase - (t_amplitude[2*(i-1)+1] - cor);
	if (Math.abs(dx) > angl)
	    if(dx > 0)
		cor -= 2.0*Math.PI;
	    else
		cor +=  2.0*Math.PI;
	phase += cor;
	t_amplitude[2*i+1] = phase;
    }
    
  // Phase shift, Group delay, Phase delay 
    for (i = 1; i < nfft/2; i++)
    {
	real = t.tr[2*i];
        imag = t.tr[2*i+1];
        amp = Math.sqrt(real*real +imag*imag);
        if(!((imag == 0.0) && (real == 0.0)))
	    phase = Math.atan2(imag,real);
        else
            phase =0.0;
        t_phase[2*i] = t_groupd[2*i] = t_phasedl[2*i] = phase;
        t_phase[2*i+1] = t_groupd[2*i+1] = t_phasedl[2*i+1] = amp;
    }
    angl = Math.PI;
    cor = 0.0;
    for (i = 2; i < nfft/2; i++)
    {
    	phase = t_phase[2*i];
        dx = phase - (t_phase[2*(i-1)] - cor);
        if (Math.abs(dx) > angl)
      	    if(dx > 0)
		cor -= 2.0*Math.PI;
            else
                cor +=  2.0*Math.PI;
        phase = phase + cor;
	t_phase[2*i] = t_groupd[2*i] = t_phasedl[2*i] = phase;
    }
     
  // Imaginary Part
    for (i=1; i < nfft/2; i++)
    {
	real = t.tr[2*i];
        imag = t.tr[2*i+1];
        t_imag[2*i] = imag;
        t_imag[2*i+1] = real;
    }
    
   // Real Part
   for (i = 0; i < nfft; i++)
	t_real[i] = t.tr[i];
    
    
   //------------ Demultiplex trace
    t_samp *= old_nfft;
    t_samp = 1.0/t_samp;
    nfft = old_nfft/2+1;
    tr_amplitude = new double [nfft];
    tr_phase = new double [nfft];
    tr_real = new double [nfft];
    tr_imag = new double [nfft];
    tr_groupd = new double [nfft];
    tr_phasedl = new double [nfft];
    
    for (i = 1; i < old_nfft/2; i++)
    {
	tr_amplitude[i] = t_amplitude[i*2];
	tr_phase[i] = t_phase[i*2];
	tr_real[i] = t_real[i*2];
	tr_imag[i] = t_imag[i*2];
	tr_groupd[i] = t_groupd[i*2];
	tr_phasedl[i] = t_phasedl[i*2];
    }
  }
 
  // DemultiCompTrace
  // -----------------
  public void DemultiCompTrace ()
  {
    double buff, buff0;
    int i;
    
    // Amplitude
    tr_amplitude[0] = t_amplitude[0];
    tr_amplitude[old_nfft/2] = Math.abs(t_amplitude[1]);
   
    // Phase shift
    tr_phase[0] = (t_phase[0] >= 0) ? 0.0 : Math.PI;
    tr_phase[old_nfft/2] = (t_phase[1] >= 0) ? 0.0 : Math.PI;
    for (i = 1; i < nfft; i++)
	tr_phase[i] *= -180/Math.PI;
    tr_phase[1] = tr_phase[2];
    tr_phase[0] = tr_phase[1];
    tr_phase[nfft-1] = tr_phase[nfft-2];
    
    // Real part
    tr_real[0] = t_real[0];
    tr_real[old_nfft/2] = t_real[1]; 
  
    // Imaginary part
    tr_imag[0] = 0;
    tr_imag[old_nfft/2] = 0;
   
    // Group delay
    tr_groupd[0] = (t_groupd[0] >= 0) ? 0.0 : Math.PI;
    tr_groupd[old_nfft/2] = (t_groupd[1] >= 0) ? 0.0 : Math.PI;
    buff0 = tr_groupd[0];
    tr_groupd[0] *= 2./((double)t_samp*2*Math.PI);
    for (i = 1; i < nfft; i++)
    {
	buff = tr_groupd[i];
	tr_groupd[i] = (tr_groupd[i] - buff0)/(t_samp*2*Math.PI);
	buff0 = buff;
    }
    /* phase for zero frequency is not defined */
    tr_groupd[1] = tr_groupd[2];
    tr_groupd[0] = tr_groupd[1];
    tr_groupd[nfft-1] = tr_groupd[nfft-2];
     
    // Phase delay
    tr_phasedl[0] = (t_phasedl[0] >= 0) ? 0.0 : Math.PI;
    tr_phasedl[old_nfft/2] = (t_phasedl[1] >= 0) ? 0.0 : Math.PI;
    for (i = 1; i < nfft; i++)
    {
	buff = t_samp* 2.0 * Math.PI * i;
	tr_phasedl[i] /= buff;
    }
    tr_phasedl[0] = tr_phasedl[1];
  } 
  
  // enablePainting 
  // ---------------
  public void enablePainting ()
  {
	amplitude = phase = real = imag = groupd = phasedl = false;
	for (int i = 0; i < nfft; i++)
	{
	    if(tr_amplitude[i] > 0) amplitude = true;
	    if(tr_phase[i] > 0) phase = true;
	    if(tr_real[i] > 0) real = true;
	    if(tr_imag[i] > 0) imag = true;
	    if(tr_groupd[i] > 0) groupd = true;
	    if(tr_phasedl[i] > 0) phasedl = true;
	}
   }
   
 // set spectrum
 //--------------
  public void setAsSpectrum (String what, int axis)
  { 
    selectResponse (what, axis);
    theLabel = spectrumSt + " - " + what;
  }
  
 // selectResponse
 // ---------------
  public void selectResponse (String what, int axtype)
  {
    axis = axtype;
    xLabel = "Hz";
    if	    (what.equals (specAmplitudeSt))  tr = tr_amplitude;
    else if (what.equals (specPhaseSt))      tr = tr_phase;
    else if (what.equals (specRealSt))	     tr = tr_real;
    else if (what.equals (specImaginarySt))  tr = tr_imag;
    else if (what.equals (specGroupDelaySt)) tr = tr_groupd;
    else if (what.equals (specPhaseDelaySt)) tr = tr_phasedl;
    theLabel = what;
    id = FREQUENCYRESPONSE;
  } 
 
 
}  