//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.lang.*;

public class WinGroup extends Thread implements ActionListener, Definitions 
{

  static String Dir_FIR = "corr_coeff/";
  static String Dir_Data = "data/";
  static String Dir_calVal = "cal/";
  static String Dir_decFIR = "decfir/";
  static String File_decFIR = "decfir.";
  static String Dir_Exercise = "exercise/";
  
  Settings settings;
  Parameter params;
  
  MainBar bar;
  SettingsEditor editor;
  WinPolesZeros PolesZeros;
  WinWarning warning;
  WinPrint printWin;
  WinMessage message;
    
  TraceSpectrum specTrace, fitSpecTrace, specGSE;
  WinShowGraph frame, winGraphI, winGraphII, winSpectrum;
  WinShowPolesZeros showPolZer;

  CalibrationValues CalVal;
  CalibrationValues FitResponseCalVal;
  Tutor tutor;
  ConnectToServer ServerConnection;
  
  int index;
    
   WinGroup (Tutor f, int index)
   {
	tutor = f;
	this.index = index;
	
	settings = new Settings (this);
	editor = new SettingsEditor (this);
	params = new Parameter ();
	
	PolesZeros = new WinPolesZeros (this);
		 
	showPolZer = new WinShowPolesZeros (this);
	winGraphI = new WinShowGraph (this,true);
	winGraphII = new WinShowGraph (this,true);
	winSpectrum = new WinShowGraph (this,true); 
	frame = new WinShowGraph (this,false);
				
	message = new WinMessage (this);
	warning = new WinWarning (this);
	printWin = new WinPrint (this);
		
	CalVal = new CalibrationValues ();
	FitResponseCalVal = new CalibrationValues ();
	ServerConnection = new ConnectToServer(this);
	
	bar = new MainBar(tutor, this);
	frame.setMenuBar (bar);
   }
   
   public void run ()
   { 
	frame.setVisible(true);
	tutor.actionPerformed (new ActionEvent (this, Event.ACTION_EVENT, "Loaded"));
   }
   
   public void actionPerformed (ActionEvent evt)
   { 
      String act = evt.getActionCommand();
	if (act.equals ("Exit"))
	{
	    hideAll();
	    disposeAll();
	    if (index == 0)
		tutor.actionPerformed (new ActionEvent (this, Event.ACTION_EVENT, "Exit"));
	    return;
	}
	if (act.equals ("Reset"))
	{ 
	    if (winGraphI.isShowing()) winGraphI.graph.Reset();
	    if (winGraphII.isShowing())winGraphII.graph.Reset();
	    hideAll();
	    PolesZeros.Reset();
	    frame.graph.Reset();
	    SetupCommand cmd = new SetupCommand (SETUPRESET, this);
	    cmd.Reset();
	    frame.setVisible (true);
	}	
	if (act.equals ("acceptNewValues")) acceptNewValues();
	else if (act.equals ("fitResponse")) FitResponse();
	else if (act.equals ("graphRepaint")) frame.graph.repaint();
   }

   public void acceptNewValues ()
   { 
	FreqResponseInitialisation();			// don't change order of first 3 if's
	if (haveingResponseType (FREQUENCYRESPONSE)) {
	    ResponseCommand resC = new ResponseCommand 
		(this, bar.responseType, bar.axisType);
	    resC.actionPerformed(new ActionEvent (this, Event.ACTION_EVENT, responseSt));
	}
	if (haveingResponseType (STEPRESPONSE)) {
	    ResponseCommand resC = new ResponseCommand 
		(this, STEPRESPONSE);
	    resC.actionPerformed(new ActionEvent (this, Event.ACTION_EVENT, responseSt));
	}
	if (haveingResponseType (IMPULSERESPONSE)) {
	    ResponseCommand resC = new ResponseCommand 
		(this, IMPULSERESPONSE);
	    resC.actionPerformed(new ActionEvent (this, Event.ACTION_EVENT, responseSt));
	}
   }	
     
   public void FreqResponseInitialisation()
   { 
	specGSE = new TraceSpectrum (new Spectrum 
		   (params.getSetupPoints(), 1/params.getSetupFrequency(), CalVal));
	specGSE.DemultiCompTrace ();
	specGSE.enablePainting();
	bar.FreqRespEnableDiagramAxis(specGSE);
   }  
     
   public void FitResponse ()
   {
	fitSpecTrace = new TraceSpectrum (new Spectrum 
	    (params.getSetupPoints(), 1/params.getSetupFrequency(), FitResponseCalVal));
	fitSpecTrace.DemultiCompTrace ();
	fitSpecTrace.enablePainting();
	bar.FreqRespEnableDiagramAxis(fitSpecTrace);
   }
     
    public void hideAll()
    {
	editor.setVisible(false);
	PolesZeros.setVisible(false);
	warning.setVisible(false);
	printWin.setVisible(false);
	message.setVisible(false);
	showPolZer.setVisible(false);
	winGraphI.setVisible(false);
	winGraphII.setVisible(false);
	winSpectrum.setVisible(false);
	frame.setVisible(false);
    }
    
    public void disposeAll()
    {
	editor.dispose();
	PolesZeros.dispose();
	warning.dispose();
	printWin.dispose();
	message.dispose();
	showPolZer.dispose();
	winGraphI.dispose();
	winGraphII.dispose();
	winSpectrum.dispose();
	frame.dispose();
    }
    
    public Graph getGraph(int what)
    {  
	if (SPECTRUM+AMPL <= what && what <= SPECTRUM+IMAG)
	    return winSpectrum.graph;
	if (FREQUENCYRESPONSE+AMPL <= what && what <= FREQUENCYRESPONSE+PHDE)
	    what = FREQUENCYRESPONSE;
	if (frame.graph.t.getId() == what) return frame.graph;
	if (winGraphI.graph.t.getId() == what) return winGraphI.graph;
	if (winGraphII.graph.t.getId() == what) return winGraphII.graph;
	if (frame.graph.t.getId() != FREQUENCYRESPONSE && 
	    frame.graph.t.getId() != IMPULSERESPONSE && frame.graph.t.getId() != STEPRESPONSE)
	   return frame.graph;
	if (winGraphI.graph.t.getId() != FREQUENCYRESPONSE && 
	    winGraphI.graph.t.getId() != IMPULSERESPONSE && winGraphI.graph.t.getId() != STEPRESPONSE)
	    return winGraphI.graph;
	return winGraphII.graph; 
    } 
 
    boolean haveingResponseType (int type)
    {
	if (FREQUENCYRESPONSE+AMPL <= type && type <= FREQUENCYRESPONSE+PHDE)
	    type = FREQUENCYRESPONSE;
	if (frame.graph.t.getId() == type ||
	    winGraphI.graph.t.getId() == type ||
	    winGraphII.graph.t.getId() == type)
	    return true;
	return false;
    }
    
    public boolean checkPoints (int points)
    {
	int nn = 2;
	while (nn < points)
	    nn *= 2;
	if (nn != points)
	    return false;
 
	// test if value is too big
	try	{
	    double g[] = new double [points];
	}
	catch (OutOfMemoryError e) {
	    return false;
	}  
	return true;
    }  
 }   
  
