//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.*;
import java.lang.*;
import java.awt.event.*;
import java.util.Stack;

public class Graph extends GraphCanvas 
implements KeyListener, Definitions
{  
    static int leftr = 55;
    static int rigthr = 40;
    static int topr = 10;
    static int downr = 40;
    static int gap = FontHeight+5;
    boolean zoom = false;
    boolean dragmode = false;
    int zoomstart, zoomstart2, zoomend;
    Point zstart, zakt, zaktbefore;
    
    boolean paint2 = false;	
    boolean both = false;
    boolean fitRes = false;
    		
    PlotTrace t;
    PlotTrace t_new;		// second PlotTrace
    PlotTrace t_orig;	
    Stack zoomedTraces;		// put in all zoomed traces 
    
    	
    // Constuctor			   		   
    Graph (WinGroup w)
    {
	super();
	setBackground(Color.white);
	wg = w;
	t = new PlotTrace();
	t_new = new PlotTrace();
	zstart = new Point(0,0);
	zakt = new Point(0,0);
	zaktbefore = new Point (0,0);
	addKeyListener (this);
	addMouseListener (this);
	addMouseMotionListener (this);
	
	zoomedTraces = new Stack();
    }
         
    public void mouseClicked (MouseEvent e) {}
    public void mouseEntered (MouseEvent e) { requestFocus(); }
    public void mouseExited (MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}
    
    public void keyTyped(KeyEvent e) {}
    public void keyPressed(KeyEvent e) {}
 
    public void keyReleased(KeyEvent e)
    {
	if (e.getKeyCode() == KeyEvent.VK_ESCAPE)	// Escape
	{
	    if (!zoomedTraces.empty())
	    {
		ZoomOutAll ();
		repaint();
	    }	    
	}
	if (e.getKeyCode() == KeyEvent.VK_F1 && !zoomedTraces.empty())	// Zoom in one step
	{
	    if (both || paint2) t_new = (PlotTrace)zoomedTraces.pop();
	    t = (PlotTrace)zoomedTraces.pop();
	    if (paint2 && !both)
		t.SameMaxMin (t_new);		
	    zoom = false;
	    repaint();
	}
    }
    
    // Zoom out all
    public void ZoomOutAll ()
    {
	do
	{
	   if (both || paint2) t_new = (PlotTrace)zoomedTraces.pop();
	   t = (PlotTrace)zoomedTraces.pop();
	}    
	while (!zoomedTraces.empty());
	if (paint2 && !both)
	    t.SameMaxMin (t_new);
    }
    
    // Mouse down
    public void mousePressed (MouseEvent e)		// Zooming is expected
    {
      PlotTrace inStack; 
      	if (e.isMetaDown()) return;	// right mouse
	if (e.getX() < leftr || e.getX() > leftr + width)   // outside trace
	{
	    zstart.x = zstart.y = -1;
	    return;
	}
	zoomedTraces.push((Object)(new PlotTrace (t, false)));
	if (both || paint2)
	   zoomedTraces.push((Object)(new PlotTrace (t_new, false)));
	if (!e.isShiftDown())				// horizontal
	{	
	    zoom = true;
	    zstart.x = zakt.x = zaktbefore.x = e.getX();
	    zstart.y = zakt.y = zaktbefore.y = e.getY();
	    zoomstart = (int)((double)(zstart.x - leftr)/(double)width * (double)t.nfft);
	    if (t.axis > 1)
	    {
		double x = (zstart.x -leftr-width/2)*t.xdiff/width +t. meanx;
		zoomstart = (int)(Math.pow (10., x)/t.t_samp);
	    }
	    if (both || paint2)
	    {
		zoomstart2 = (int)((double)(zstart.x - leftr)/(double)width * (double)t_new.nfft);
		if (t_new.axis > 1)
		{
		    double x = (zstart.x -leftr-width/2)*t_new.xdiff/width +t_new.meanx;
		    zoomstart2 = (int)(Math.pow (10., x)/t_new.t_samp);
		}
	    }
	}
    }
    
    // Mouse drag
   public void mouseDragged(MouseEvent e)
   {	
	if(e.isShiftDown()) return;
	if (zstart.x == -1)
	    return;
	zoom = true;
	dragmode = true;
	zaktbefore.x = zakt.x;
	zaktbefore.y = zakt.y;
	zakt.x = e.getX();
	zakt.y = e.getY();
	zoomend = (int)((double)(zakt.x - leftr)/(double)width * (double)t.nfft);
	repaint();
    }
    
    // Mouse up
    public void mouseReleased (MouseEvent e)
    {
	if (e.isMetaDown())		// right mouse, zoom out
	{
	    if (!zoomedTraces.empty())
	    {
		ZoomOutAll ();
		repaint();
		return;
	    }
	}
	if(zoomedTraces.empty()) return;
	if (zstart.x == -1) return;
	if(e.isShiftDown())		// vertical zoom
	{
	    t.VerticalZoom();
	    if (both || paint2)
		t_new.VerticalZoom();
	    repaint();	
	} 
	else				// horizontal zoom
	{
	    zakt.x = e.getX();
	    zakt.y = e.getY(); 
	    if (dragmode)
	    {
		if (t.axis > 1)
		{
		    double x = (zakt.x -leftr-width/2)*t.xdiff/width +t. meanx;
		    zoomend = (int)(Math.pow (10., x)/t.t_samp);
		}
		else
		    zoomend = (int)((double)(zakt.x - leftr)/(double)width * (double)t.nfft);
		if (zoomstart == zoomend) return;	// sometimes error occurs
		t.ZoomTrace(zoomstart, zoomend);
		if (both || paint2)
		{
		    if (t.axis > 1)
		    {
			double x = (zakt.x -leftr-width/2)*t.xdiff/width +t. meanx;
			zoomend = (int)(Math.pow (10., x)/t.t_samp);
		    }
		    else
			zoomend = (int)((double)(zakt.x - leftr)/(double)width * (double)t_new.nfft);
		    t_new.ZoomTrace(zoomstart2, zoomend); 
		}
		if (paint2 && !both)
		    t.SameMaxMin (t_new);
	    }	
	    dragmode = false;	
	    repaint();
	}    
	zoom = false;
    }
 
    // SetPlotTrace  
    public void SetPlotTrace (PlotTrace T/*, int modus*/)
    {
	if (!zoomedTraces.empty())	// empty stack
	    do
		t = (PlotTrace)zoomedTraces.pop(); 
	    while (!zoomedTraces.empty());
	t = new PlotTrace (T, true);
	paint2 = false;
	both = false;
	
      // Preperation for spectrum	
	if (SPECTRUM+AMPL <= t.getId() && t.getId() <= SPECTRUM+IMAG)
	    return;
	
	wg.specTrace = new TraceSpectrum (new PlotTrace 
		    (t, !wg.checkPoints(wg.params.getSetupPoints())? 
		    wg.params.getSetupDefaultPoints() : wg.params.getSetupPoints()));
	wg.specTrace.enablePainting();
	wg.bar.SpectrumEnableDiagramAxis(wg.specTrace);   
    }
     
    // SetSecondPlotTrace		
    public void SetSecondPlotTrace (PlotTrace T, boolean Both)
    {
	t_new = new PlotTrace (T, true);
	if (!Both)   // plot into the same window
	    t.SameMaxMin (t_new);
	paint2 = true;
	both = Both;
    }  
    
    // DeleteSecondPlotTrace
    public void DeleteSecondPlotTrace ()
    {
	t_new.tr = null;
	paint2 = false;
	t.MaxMin ();
    }  

    // Save PlotTrace t 
    public void SavePlotTrace ()
    {
	t_orig = new PlotTrace (t, true);
    }
    
    // Reset
    public void Reset()
    {
	if (!zoomedTraces.empty())	// empty stack
	    do
		t = (PlotTrace)zoomedTraces.pop();
	    while (!zoomedTraces.empty());
	t = t_new = new PlotTrace();
	setLabel ("");
	Graphics g = getGraphics();
	g.clearRect(0,0,getSize().width, getSize().height);
    } 
    
 //************* Update  **************   
  public void update (Graphics g)
  { 
	image = createImage (getSize().width, getSize().height);
	Graphics imgGraphics = image.getGraphics();
	imgGraphics.setColor (cBackground);
	imgGraphics.fillRect (0, 0, getSize().width, getSize().height);
	paint (imgGraphics);
	g.drawImage (image, 0, 0, this);
  }
  
  //************* Paint **************   
   public void paint (Graphics g)
   {  
	if (t.getId() == -1)
	    return;
	widthAndHeight();
        setSize (getSize().width, getSize().height);
	if (!isVisible())setVisible (true);
	
	if ((t.axis == 1 || t.axis == 3) && !checkPainting())
	{
	    g.drawString ("not possible", pwidth/2, pheight/2);
	    return;
	}
	
	g.setColor(cLines);
	g.drawRect (leftr, topr,width,height);  // bolder
	if (both) g.drawRect (leftr, topr+deltaH,width,height2);  // bolder
	
	// zooming area
	if (zoom == true)
        {
	    g.setColor(cZoom);
	    int st = (zstart.x < zakt.x) ? zstart.x : zakt.x;
	    int en = (zstart.x < zakt.x) ? zakt.x : zstart.x;
            g.fillRect (st, topr+1, en - st, height-1);
	    if (both)
	        g.fillRect (st, topr+deltaH+1, en-st, height2-1);
	    g.setColor(cLines);
            String str = valStr(zoomstart*t.t_samp)+"... "+valStr(zoomend*t.t_samp)+": "+
                         valStr(zoomend*t.t_samp-zoomstart*t.t_samp)+" "+t.xLabel;
            g.drawString (str, width/2, topr);
            zoom = false;
        }
	plot_X_axis (g,t,width,height,0);
	plot_Y_axis (g,t,width,height,0);
	g.setColor(cPoints);	
	plot_Trace (g,t,width,height,0);
	if (paint2 && !both)
	{ 
	    g.setColor(cPoints_new);
	    plot_Trace (g, t_new, width, height, 0);
	    setLabel (t_new.theLabel);
	}	
    	else if (paint2 && both)
	{
	    plot_X_axis (g,t_new,width,height,deltaH);
	    plot_Y_axis (g,t_new,width,height,deltaH);
	    g.setColor(cPoints_new);
	    plot_Trace (g, t_new, width, height, deltaH);
	    setLabel (t_new.theLabel);
	}
	else 
	   setLabel (t.theLabel);
   }
  
   protected boolean checkPainting()
   {
	for (int i = 0; i < t.nfft; i++)
	    if (t.tr[i] > 0)
		return (true);
	return (false);
   }
    
   protected void plot_Trace (Graphics g, PlotTrace T, int w, int h, int deltah)
   {
    int i;
    int x1, x2, y1, y2;
    double xx1, xx2, yy1, yy2;
      if (!T.discr)	
	for (i = 0; i < T.nfft-1; i++)
	{   
	    xx1 = i*T.t_samp + T.zoom_start*T.t_samp + T.old_zoom_start*T.t_samp;
	    xx2 = (i+1)*T.t_samp + T.zoom_start*T.t_samp + T.old_zoom_start*T.t_samp;
	    if (T.axis == 2 || T.axis == 3)	    // x log
	    {
		if (xx1 <= 0 && xx2 <= 0)
		    continue;
		if (xx1 <= 0 && xx2 > 0)
		    xx1 = xx2;
		else if (xx1 > 0 && xx2 <= 0)
		    xx1 = xx1;
		xx1 = 0.4343*Math.log(xx1);
		xx2 = 0.4343*Math.log(xx2);    
	    }		    
	    x1 = (int)((double)(leftr+w/2+(double)w/T.xdiff*(xx1-T.meanx)));	
	    x2 = (int)((double)(leftr+w/2+(double)w/T.xdiff*(xx2-T.meanx)));	
	    if ((T.axis == 1 || T.axis == 3) &&
	        (T.tr[i] <= 0 || T.tr[i+1] <= 0))
		continue;
	    yy1 = (T.axis == 1 || T.axis == 3)
		   ? 0.4343*Math.log(T.tr[i]) : T.tr[i];
	    yy2 = (T.axis == 1 || T.axis == 3)  
		   ? 0.4343*Math.log(T.tr[i+1]) : T.tr[i+1];
	   
	    y1 = deltah + (int)((double)((double)(topr+h/2.0)-(double)h/
		    T.ydiff*(yy1-T.meany)));	
	    y2 = deltah + (int)((double)((double)(topr+h/2.0)-(double)h/
		    T.ydiff*(yy2-T.meany)));	
	    g.drawLine (x1,y1,x2,y2);
	}
      else
      	for (i = 1; i < T.nfft; i++)
	{ 
	  x1 = (int)((double)(leftr+w/2+(double)w/
		T.xdiff*(i*T.t_samp+ T.zoom_start*T.t_samp + T.old_zoom_start*T.t_samp-T.meanx)));
	  y1 = deltah + (int)((double)(topr+h/2.0-(double)h/T.ydiff*(-T.meany)));
	  x2 = (int)((double)(leftr+w/2+(double)w/
	    	T.xdiff*((T.tr[i]==0.0 ? i-1:i)*T.t_samp+ T.zoom_start*T.t_samp + T.old_zoom_start*T.t_samp-T.meanx)));
	  y2 = deltah+(int)((double)(topr+h/2.0-(double)h/T.ydiff*((T.tr[i]==0.0?0:T.tr[i])-T.meany)));
	  g.drawLine (x1,y1,x2,y2);
	  g.drawLine (x1+1,y1,x2+1,y2);
	 }
  }
  
  protected void widthAndHeight()
  {
	height2 = deltaH = 0;
	pwidth = getSize().width;
	pheight= getSize().height;
	width = pwidth-(leftr + rigthr);
	if (both)
	{
	    height = height2 = (pheight- topr - downr - gap)/2;
	    deltaH = topr + height + gap;
	}
	else
	    height = pheight-(topr + downr);
  }
  
  protected void plot_X_axis(Graphics g, PlotTrace T, int w, int h, int deltah)
  {
    double space, x, xx1;
    int x1;
   	if (T.axis == 0 || T.axis == 1)			// x lin
	{
	    space = FindSpace ((T.xdiff < 1) ? T.xdiff : Math.round(T.xdiff));
	    for(x = T.xmax-(T.xmax%space); x >= T.xmin; x -= space)
	    {  
	  	g.setColor(cKoordLines);
		x1 = (int)((double)(leftr+w/2+(double)w/(double)T.xdiff*(x-T.meanx)));	
		g.drawLine(x1,topr+deltah,x1,topr+h+deltah);
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(x),x1-5,topr+h+deltah+FontHeight+2);
	    }
	}  
	else					    // x log
	{
	    axis_log(0,g,leftr,topr+h, leftr+w,topr, T.xmin,T.xmax,w,h); 
	    for (x = t.xmin; x < t.xmax+0.5; x += 1)
	    {
		xx1 = Math.pow(10.0,(double)x);
		x1 = (int)((double)(leftr+w/2+(double)w/(double)T.xdiff*(x-T.meanx)));
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(xx1),x1-5,topr+h+deltah+FontHeight+2);
	    } 
	}
	g.drawString (T.xLabel,leftr+w+rigthr/2,topr+h+deltah+FontHeight+2);
  }
  
  protected void plot_Y_axis (Graphics g, PlotTrace T, int w, int h, int deltah)
  {
    double space, y, yy1;
    int y1 = 0, y2;
    int first = 0;
    boolean havefirst = false;
	int ymin = deltah + (int)((double)((double)(topr+h/2.0)-(double)h/
		   T.ydiff*(T.ymin-T.meany)));	
	int ymax = deltah + (int)((double)((double)(topr+h/2.0)-(double)h/
		   T.ydiff*(T.ymax-T.meany)));
  	if (T.axis == 0 || T.axis == 2)
	{
	    space = FindSpace ((T.ydiff < 1) ? T.ydiff : Math.round(T.ydiff));
	    for( y = T.ymax-(T.ymax%space); y > T.ymin ; y -= space)
	    { 

		g.setColor(cKoordLines);
		y1 = (int)((double)(topr+h/2.0-(double)h/T.ydiff*(y-T.meany)));
		if (!havefirst)
		{
		    havefirst = true;
		    first = y1;
		}
		if (y1+deltah > topr) g.drawLine(leftr,y1+deltah,leftr+w,y1+deltah);
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(y),5,y1+deltah+FontHeight/2);
	    }
	    if (first+deltah - ymax > FontHeight/2)
	    {
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(T.ymax),5,ymax+FontHeight/2);
	    }
	    if (ymin+deltah - y1 > FontHeight/2)
	    {
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(T.ymin),5,ymin+FontHeight/2);
	    }
	}
	else
	{
	    axis_log(1,g,leftr,topr+h, leftr+w,topr, T.ymin,T.ymax,w,h); 
	    for (y2 = 0,y = T.ymin; y < T.ymax+0.5; y += 1)
	    {
		yy1 = Math.pow(10.0,(double)y);
		y1 = (int)((double)(topr+h/2+(double)h/(double)T.ydiff*(y-T.meany)));
		if (y == t.ymin) y2 = y1;
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(yy1),5,topr+h+y2-y1+deltah+FontHeight/2);
	    } 
	}   
  }
 
  protected void axis_log(int xaxis, Graphics g,int x1,int y1, int x2, int y2,
			  double uu1, double uu2, int w, int h)
  {
    int nodecs; 
    int i;
    int noticks;
    int xold,yold,xnew,ynew;
    int count;
    double xinc,yinc;
    double value,valinc;
    double scale; 
    nodecs=(int)Math.abs((uu2-uu1));

    g.setColor(cKoordLines);
    noticks=9*nodecs;
    if (xaxis == 1)
    {
	scale=(float)(y2-y1)/nodecs;
	for(valinc = value = 1.0, count = i = 0; i< noticks-1;i++)
        {
	    count+=1;
            if (count >= 10)
            {
		count=1;
                valinc=valinc*10.0;
            }
            value+=valinc;
            yinc= (int)(0.4343*Math.log(value)*scale);
            yold= y1 + (int)yinc;
	    g.drawLine (leftr,yold,leftr+w,yold);
        }
    }
    else
    {  
	yold=y1;
        ynew= yold - h;
        scale=(float)(x2-x1)/nodecs;
        for(valinc = value = 1.0, count = i = 0;i < noticks-1;i++)
        {
	    count+=1;
            if (count >= 10)
            {
		count=1;
		valinc=valinc*10.0;
            }
            value+=valinc;
            xinc= (int)(0.4343*Math.log(value)*scale);
            xold= x1 + (int)xinc;
	    g.drawLine (xold,yold,xold,ynew);
	} 
    }	
  } 
 
}    
