//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.*;
import java.lang.*;
import java.awt.event.*;

public class PZGraph extends GraphCanvas
{
    static int leftr = 10;
    static int rigthr = 10;
    static int topr = 10; 
    static int downr = 10;
    int BIG = 8;
    int ArrowLen = 10;	    
    
    float xmax, xmin;	// real part
    float ymax, ymin;	// imaginary part
    double ydiff, meany;		
    double xdiff, meanx;	
    	
    Rectangle GPoles[] = null;
    Rectangle GZeros[] = null;
    Rectangle KPoles[] = null;
    Rectangle KZeros[] = null;
    double RadPoles[] = null;	    // radius of pol circles
    double RadZeros[] = null;
    Point NP;			    // x,y values of null point of the coordinate system 
    int handlingZeros[];	    // marks handled zeros, 1 = handled pole, 2 = pair pole
    int handlingPoles[];	    // marks handled poles, 1 = handled pole, 2 = pair pole
   
    boolean isShowing = false;
    
    boolean dragMode = false;
    Image img;
    int paintStatus = 0;	    // 0 normal, 1 drag, 2 drag/down + shift(circles)
      
    PZGraph (WinGroup w)
    {
	setBackground(cBackground);
	wg = w;
    	addMouseListener (this);
	addMouseMotionListener (this);
    }
    
    public void mouseClicked (MouseEvent e) {}   
    public void mouseEntered (MouseEvent e) { requestFocus(); }
    public void mouseExited (MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}
    
    public void mousePressed (MouseEvent e)  
    {
      int i, j;
	paintStatus = 0;
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	    if (GZeros[i].contains(e.getX(),e.getY()))
	    {
		if (e.isShiftDown()) paintStatus = 2;
		handlingZeros[i] = 1;
		if (wg.CalVal.paarZeros[i] > 0)
		    for (j = 0; j < wg.CalVal.numOfZeros; j++)
			if ((wg.CalVal.paarZeros[i] == wg.CalVal.paarZeros[j]) && i != j)
			     handlingZeros[j] = 2;
		break;			     
	    }	
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	    if (GPoles[i].contains(e.getX(),e.getY()))
	    {
		if (e.isShiftDown()) paintStatus = 2;
		handlingPoles[i] = 1;
		if (wg.CalVal.paarPoles[i] > 0)
		    for (j = 0; j < wg.CalVal.numOfPoles; j++)
			if ((wg.CalVal.paarPoles[i] == wg.CalVal.paarPoles[j]) && i != j)
			     handlingPoles[j] = 2;
		break;		     
	    }			     
	if (paintStatus == 2) repaint();   // to plot the circle  
    }
    
    public void mouseDragged(MouseEvent e)
    {
      int i, x, y;
	dragMode = true;
	paintStatus = 1;  
	x = e.getX(); y = e.getY(); 
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	    if (handlingZeros[i] == 1 || handlingZeros[i] == 2)
	    {
	   	if (e.isShiftDown() && wg.CalVal.paarZeros[i] > 0)	    
		{
		    paintStatus = 2;
		    Point p = PointatCircle(KZeros[i].width/2, x, y);
		    x = p.x;
		    y = p.y;
		}	
		wg.CalVal.zeros[i].r = x_to_real(x);
		wg.CalVal.zeros[i].i = y_to_imaginary(y);
		wg.CalVal.zeros[i].i = (handlingZeros[i] == 1) ? y_to_imaginary(y) : -y_to_imaginary(y);
	    }
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	    if (handlingPoles[i] == 1 || handlingPoles[i] == 2)
	    {
	   	if (e.isShiftDown() && wg.CalVal.paarPoles[i] > 0)	    
		{
		    paintStatus = 2;
		    Point p = PointatCircle(KPoles[i].width/2, x, y);
		    x = p.x;
		    y = p.y;
		}	
		wg.CalVal.poles[i].r = x_to_real(x);
		wg.CalVal.poles[i].i = y_to_imaginary(y);
		wg.CalVal.poles[i].i = (handlingPoles[i] == 1) ? y_to_imaginary(y) : -y_to_imaginary(y);
	    } 
	repaint();  
    }	
    
    public void mouseReleased (MouseEvent e)
    {
	int i, x, y;
	x = e.getX(); y = e.getY();
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	    if (handlingZeros[i] == 1)
	    {
		if (paintStatus == 2)
		{
		    Point p = PointatCircle(KZeros[i].width/2, x, y);
		    x = p.x;
		    y = p.y;
		}	
		wg.CalVal.zeros[i].r = x_to_real(x);
		wg.CalVal.zeros[i].i = (handlingZeros[i] == 1) ? y_to_imaginary(y) : -y_to_imaginary(y);
		handlingZeros[i] = 0;
	    }	
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	    if (handlingZeros[i] == 2)
	    {
		wg.CalVal.zeros[i].r = x_to_real(x);
		wg.CalVal.zeros[i].i = (handlingZeros[i] == 1) ? y_to_imaginary(y) : -y_to_imaginary(y);
		handlingZeros[i] = 0;
	    }	
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	    if (handlingPoles[i] == 1)
	    {
		if (paintStatus == 2)
		{
		    Point p = PointatCircle(KPoles[i].width/2, x, y);
		    x = p.x;
		    y = p.y;
		}	
		wg.CalVal.poles[i].r = x_to_real(x);
		wg.CalVal.poles[i].i = (handlingPoles[i] == 1) ? y_to_imaginary(y) : -y_to_imaginary(y);
		handlingPoles[i] = 0;
	    }	
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	    if (handlingPoles[i] == 2)
	    {
		wg.CalVal.poles[i].r = x_to_real(x);
		wg.CalVal.poles[i].i = (handlingPoles[i] == 1) ? y_to_imaginary(y) : -y_to_imaginary(y);
		handlingPoles[i] = 0;
	    }	
	paintStatus += 10;
	repaint();
    }
    
  public void update (Graphics g)
  {
	image = createImage (getSize().width, getSize().height);
	Graphics imgGraphics = image.getGraphics();
	imgGraphics.setColor (cBackground);
	imgGraphics.fillRect (0, 0, getSize().width, getSize().height);
	paint (imgGraphics);
	g.drawImage (image, 0, 0, this);
  }
     
    public void paint (Graphics g)
    {  
      int i;
      int x,y;
      	isShowing = true;
	
	pwidth = getSize().width;
	pheight= getSize().height;
	g.clearRect(0,0,pwidth, pheight);
	width = pwidth-(leftr + rigthr);
	height = pheight-(topr + downr);
	if (width < height) height = width;	// do this to get a round circel
	else width = height;
	
	// coordinate system
	g.setColor(cLines);
	g.drawLine (leftr+width/2, topr, leftr+width/2, topr+height);	    // y-axis
	g.drawLine (leftr, topr+height/2, leftr+width, topr+height/2);	    // x-axis
	NP = new Point (leftr+width/2,topr+height/2);
	// arrows
	g.drawLine (leftr+width,topr+height/2,leftr+width-ArrowLen,topr+height/2+ArrowLen/4); //x
	g.drawLine (leftr+width,topr+height/2,leftr+width-ArrowLen,topr+height/2-ArrowLen/4);
	g.drawLine (leftr+width/2,topr,leftr+width/2-ArrowLen/4,topr+ArrowLen);		      //y
	g.drawLine (leftr+width/2,topr,leftr+width/2+ArrowLen/4,topr+ArrowLen);
	// labels
	g.drawString("real",leftr+width-25,leftr+height/2-2-FontHeight);
	g.drawString("imaginary",leftr+width/2+25,topr+FontHeight);
	
	if (paintStatus == 0 || paintStatus == 11) MaxMin();
	ScaleAxis(g);
	
	Poles_Zeros_Koordinates();
	
	// Poles
	g.setColor(cPoles);
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	{
	    g.drawLine (GPoles[i].x, GPoles[i].y, GPoles[i].x+BIG, GPoles[i].y+BIG);
	    g.drawLine (GPoles[i].x, GPoles[i].y+BIG, GPoles[i].x+BIG, GPoles[i].y);
	    g.drawLine (GPoles[i].x-1, GPoles[i].y, GPoles[i].x+BIG-1, GPoles[i].y+BIG);
	    g.drawLine (GPoles[i].x-1, GPoles[i].y+BIG, GPoles[i].x+BIG-1, GPoles[i].y);
	    if (paintStatus == 2 || paintStatus > 11)
		if (handlingPoles[i] == 1)
		    g.drawOval (KPoles[i].x, KPoles[i].y, KPoles[i].width, KPoles[i].height);
	}    
	  
	// Zeros
	g.setColor(cZeros);
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	{
	    g.drawOval (GZeros[i].x, GZeros[i].y, GZeros[i].width, GZeros[i].height);
	    g.drawOval (GZeros[i].x-1, GZeros[i].y-1, GZeros[i].width+2, GZeros[i].height+2);
	    if (paintStatus == 2 || paintStatus > 11)
		if (handlingZeros[i] == 1)
		    g.drawOval (KZeros[i].x, KZeros[i].y, KZeros[i].width, KZeros[i].height);
	}   
	    
	if (paintStatus > 10) paintStatus = 0;	    // end of shift+drag	
    }	

    protected void  G_Initalize_Poles_Zeros()
    {
	int i;
	GPoles = new Rectangle[wg.CalVal.numOfPoles];
	KPoles = new Rectangle[wg.CalVal.numOfPoles];
	RadPoles = new double [wg.CalVal.numOfPoles];
	GZeros = new Rectangle[wg.CalVal.numOfZeros];
	KZeros = new Rectangle[wg.CalVal.numOfZeros];
	RadZeros = new double [wg.CalVal.numOfZeros];
	handlingPoles = new int[wg.CalVal.numOfPoles];
	handlingZeros = new int[wg.CalVal.numOfZeros];
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	    handlingPoles[i] = 0;
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	     handlingZeros[i] = 0;    
    }
	
    protected void Poles_Zeros_Koordinates()	
    {
	int i,x,y,xr,yr;
	// Poles
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	{   
	    x = (int)((float)(leftr+width/2.0+(float)width/
		    xdiff*(wg.CalVal.poles[i].r - meanx)));
	    y = (int)((float)(topr+height/2.0-(float)height/
		    ydiff*(wg.CalVal.poles[i].i - meany)));	
	    GPoles[i] = new Rectangle (x-BIG/2,y-BIG/2,BIG,BIG);
	    // Circles for complex conjugate pairs
	    if (paintStatus != 2 && paintStatus != 12)	
		if (wg.CalVal.paarPoles[i] > 0)
		{
		    RadPoles[i] = Math.sqrt (wg.CalVal.poles[i].r*wg.CalVal.poles[i].r +
					  wg.CalVal.poles[i].i*wg.CalVal.poles[i].i); 
		    x = (int)((float)(leftr+width/2.0+(float)width/
			xdiff*(-RadPoles[i] - meanx)));
		    xr =  (int)((float)(leftr+width/2.0+(float)width/
			xdiff*(RadPoles[i] - meanx)));   
		    y = (int)((float)(topr+height/2.0-(float)height/
			ydiff*(RadPoles[i] - meany)));
		    yr = (int)((float)(topr+height/2.0-(float)height/
			ydiff*(-RadPoles[i] - meany)));
		    KPoles[i] =  new Rectangle  (x,y,xr-x,yr-y); 
		}	
		else 
		{
		    KPoles[i] =  new Rectangle  (0,0,0,0);
		    RadPoles[i] = 0;
		}     
	}
	
	// Zeros
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	{
	    x = (int)((float)(leftr+width/2.0+(float)width/
		    xdiff*(wg.CalVal.zeros[i].r - meanx)));
	    y = (int)((float)(topr+height/2.0-(float)height/
		    ydiff*(wg.CalVal.zeros[i].i - meany)));	
	    GZeros[i] = new Rectangle (x-BIG/2,y-BIG/2,BIG,BIG);	
	    // Circles for complex conjugate pairs
	    if (paintStatus != 2 && paintStatus != 12)	
		if (wg.CalVal.paarZeros[i] > 0)
		{
		    RadZeros[i] = Math.sqrt (wg.CalVal.zeros[i].r*wg.CalVal.zeros[i].r +
					  wg.CalVal.zeros[i].i*wg.CalVal.zeros[i].i); 
		    x = (int)((float)(leftr+width/2.0+(float)width/
			xdiff*(-RadZeros[i] - meanx)));
		    xr =  (int)((float)(leftr+width/2.0+(float)width/
			xdiff*(RadZeros[i] - meanx)));   
		    y = (int)((float)(topr+height/2.0-(float)height/
			ydiff*(RadZeros[i] - meany)));
		    yr = (int)((float)(topr+height/2.0-(float)height/
			ydiff*(-RadZeros[i] - meany)));
		    KZeros[i] =  new Rectangle  (x,y,xr-x,yr-y); 
		}	
		else 
		{
		    KZeros[i] =  new Rectangle  (0,0,0,0);
		    RadZeros[i] = 0;
		}     
	}
    }
    
    protected Point PointatCircle (int w, int x, int y)
    {
      double r;
      int xp, yp;
   	
	xp = x-NP.x;
	yp = NP.y-y;
	r = Math.sqrt (xp*xp + yp*yp);
	if (r != 0) return new Point (xp*w/(int)r + NP.x, NP.y - yp*w/(int)r);
	else	    return new Point (0,0);
    } 
    
    float x_to_real (int x)
    {
	return ((float)(meanx+xdiff/(float)width*(float)(x-leftr-width/2)));
    }
    
    float y_to_imaginary (int y)
    {
	return ((float)(meany-ydiff/(float)height*(float)(y-topr-height/2)));
    }
    	       
    void MaxMin ()
    {
       int i;
	
	xmax = Math.abs(wg.CalVal.poles[0].r);
	for (i = 0; i < wg.CalVal.numOfPoles; i++)
	{
	    if (Math.abs(wg.CalVal.poles[i].r) > xmax) xmax = Math.abs(wg.CalVal.poles[i].r);
	    if (Math.abs(wg.CalVal.poles[i].i) > xmax) xmax = Math.abs(wg.CalVal.poles[i].i);
	}
	for (i = 0; i < wg.CalVal.numOfZeros; i++)
	{
	    if (Math.abs(wg.CalVal.zeros[i].r) > xmax) xmax = Math.abs(wg.CalVal.zeros[i].r); 
	    if (Math.abs(wg.CalVal.zeros[i].i) > xmax) xmax = Math.abs(wg.CalVal.zeros[i].i);
	} 
	xmax += xmax == 0 ? 1 : xmax;
	ymax = xmax;
	xmin = ymin = -xmax;
	xdiff = ydiff = xmax - xmin;
	meanx = meany = (xmax + xmin)/2.0;
    }	
	
    void ScaleAxis (Graphics g)
    {
       double space, w, xx1;
	int w1;
	 space = 2*FindSpace (xdiff/2);
	 for(w = xmax-(xmax%space); w >= xmin; w -= space)
	 {
	    g.setColor(cLines);
	    w1 = (int)((double)(leftr+width/2+(double)width/
		    (double)xdiff*(w-meanx)));
	    if (w1 < leftr+width-ArrowLen)
	    {	    	
		g.drawLine(w1,topr+height/2-5,w1,topr+height/2+5);
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(w),w1-5,topr+height/2+5+FontHeight+2);
	    }
	}
	
	space = 2*FindSpace (ydiff/2);
	for(w = ymax-(ymax%space); w >= ymin ; w -= space)
	{
	    g.setColor(cLines);
	    w1 = (int)((double)(topr+height/2.0-(double)height/
		ydiff*(w-meany)));
	    if (w1 > topr+ArrowLen)
	    {	
		g.drawLine(leftr+width/2-5,w1,leftr+width/2+5,w1);
		g.setColor(cLineText);
		g.setFont (fSmall);
		g.drawString(valStr(w),leftr+width/2+5+2,w1+FontHeight/2);
	    }
	}
    }	
  
  }    