//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.event.*;
import java.awt.*;
import java.util.*;

class ProccedCommand extends Command
{
    PlotTrace origTrace, store;
    boolean storeStore;
    
    ProceedSignal proceedSignal, proceedSignal_before;
    
    int no_it2,no_it3,no_it5;	    // Interpolation, no of iterations 
    
    public ProccedCommand (int id, WinGroup wg)
    {
	super (id, wg);
	proceedSignal = new ProceedSignal(wg);
    }
    
    public void actionPerformed (ActionEvent e)
    {
	wg.bar.Reconstruction.setEnabled (false);
	boolean twoPlots = false;;
	String s = e.getActionCommand();
	origTrace = new PlotTrace (wg.frame.graph.t, false);
	proceedSignal_before = proceedSignal;
	proceedSignal = new ProceedSignal (wg, origTrace, id);
	
	if (s.equals (cancelSt)) {
	    wg.editor.setVisible(false);
	    wg.frame.graph.SetPlotTrace (origTrace);
	    wg.actionPerformed (new ActionEvent (this, Event.ACTION_EVENT, "graphRepaint")); 
	    return;
	}
	
	switch (id) {
	
	    case MOVINGAVERAGE: {
		twoPlots = false;
		double value = wg.editor.getDouble (movingAverageWindowSt);
		if (value != Double.NEGATIVE_INFINITY) { 
		    if (value < 0) {
			wg.message.setVisible ("No valid width !");
			return;
		    } 
		    if (wg.params.saveParameter (this, value))
			proceedSignal.MovingAverage(value);
		    else proceedSignal = proceedSignal_before;
			
		} else return;
		break;
	    }
	    
	    case INTERPOLATE: {
		twoPlots = true;
		int ipol_fac = wg.editor.getInteger (interpolationFactorSt);
		if (ipol_fac != Integer.MIN_VALUE) {
		    if (ipol_fac == 1) {	
			wg.message.setVisible ("Nothing to interpolate");
			return;	
		    }
		    if (!getInterpolationFactors (ipol_fac)) {
			wg.message.setVisible (ipol_fac+" cannot be separated in factors 2, 3, and 5");
			return;
		    }
		    if (wg.params.saveParameter (this, ipol_fac))
			proceedSignal.Interpolate (ipol_fac,no_it2,no_it3,no_it5);
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
	    
	    case DISCRETIZATION:
		wg.bar.Reconstruction.setEnabled (true);
	    case RECONSTRUCTION: {
		twoPlots = false;
		double value = wg.editor.getDouble (discretizationFrequencySt);
		if (value != Double.NEGATIVE_INFINITY) { 
		    if (value < 0) {
			wg.message.setVisible ("No valid frequency !");
			return;
		    } 
		    if (wg.params.saveParameter (this, value)) {
			if (id == DISCRETIZATION) proceedSignal.discretisation (value);
			if (id == RECONSTRUCTION) proceedSignal.reconstruction (value);
		    }
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
	     
	    case DECIMATE: {
		twoPlots = true;
		int fac = wg.editor.getInteger (decimateFactorSt);
		int force = wg.editor.getInteger (decimateForceSt);
		if (fac != Integer.MIN_VALUE && force != Integer.MIN_VALUE) {
		    if (fac < 0 || force < 0) {
			wg.message.setVisible ("No valid values !");
			return;
		    }
		    if (wg.params.saveParameter(this, fac, force))
			 proceedSignal.Decimation (fac,force);
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
	    
	    case INTEGRATION:
	    case PLAINSUMMATION: {
		proceedSignal.MakeIntegrate ();
		break;
	    }
	    
	    case DIFFERENTIATE: {
		proceedSignal.Differentiate ();
		break;
	    }
	    case PLAINADCQUANTIZED:
	    case PLAINADCMANTISSA:
	    case PLAINADCEXPONENT:
	    case PLAINADCERROR: {
		double freq = wg.editor.getDouble (plainADCFrequencySt + wg.params.getSetupFrequency() + " [Hz]");
		double gain = wg.editor.getDouble (plainADCGain2St);
		double LSBvalue =  wg.editor.getDouble (plainADCLSBSt);
		int mant = wg.editor.getInteger (plainADCMantissa2St);
		int exp = wg.editor.getInteger (plainADCExponent2St);
		if (mant != Integer.MIN_VALUE && exp != Integer.MIN_VALUE &&
		    freq != Double.NEGATIVE_INFINITY && mant != Double.NEGATIVE_INFINITY &&
		    LSBvalue != Double.NEGATIVE_INFINITY) {
		    if (freq <= 0) {
			wg.message.setVisible ("No valid frequency !");
			return;
		    }
		    if (mant <= 0 || mant > 30) {
			wg.message.setVisible ("No valid resolution !");
			return;
		    }	
		    if (LSBvalue <= 0) {
			wg.message.setVisible ("No valid LSB value !");
			return;
		    }	
		    if (exp < 0) {
			wg.message.setVisible ("No valid no. of bits for exponent !");
			return;
		    }
		    if (wg.params.saveParameter (this, freq, gain, LSBvalue) ||
			wg.params.saveParameter (this, mant, exp))
			proceedSignal.plainADC (1/freq, gain, mant, exp, LSBvalue);
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
	    
	    case DECIMATIONFILTERING: {
		twoPlots = false;
		int decratio = (Integer.valueOf(wg.editor.getSelection(decimationRatioSt))).
				intValue();
		String resamp = wg.editor.getSelection(decimationFilteringResampSt);
		boolean b1 = wg.params.saveParameter (this, decratio);
		boolean b2 = wg.params.saveParameter (this, resamp);
		if (b1 || b2)
		     proceedSignal.DecimationFiltering(decratio, resamp);
		else proceedSignal = proceedSignal_before;
		break;
	    }
	    
	    case FIR2CAUS: {
		twoPlots = true;
		String file = wg.editor.getSelectedItem (fir2causCoorFilterFileSt);
		double freq = wg.editor.getDouble (fir2causFdigSt);
		int number = wg.editor.getInteger (fir2causZerosAppendSt); 
		boolean correct_time = wg.editor.getSelection(fir2causCorrectLinearPhasesSt).
					equals (fir2causCorrectLinearPhaseYesSt) ? true : false;
		if (freq != Double.NEGATIVE_INFINITY && number != Integer.MIN_VALUE && file != null) {
		    if (freq < 0 && freq != -1) {
			wg.message.setVisible ("No valid frequency !");
			return;
		    }
		    if (number < 0 ) {
			wg.message.setVisible ("No valid phase !");
			return;
		    }
		    if (wg.params.saveParameter (this, file) ||
			wg.params.saveParameter (this, freq) || 
			wg.params.saveParameter (this, number) ||
			wg.params.saveParameter (this, correct_time))
			proceedSignal.fir2caus (file,freq,number,correct_time);
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
	    
	    case DIFFERENCEEQUATION: {
		double a[], b[];
		boolean inv = wg.editor.getState (differenceEquationInvSt);
		double gain = wg.editor.getDouble (differenceEquationGainSt);
		Vector maCoef = wg.editor.getVector (differenceEquationMACoefSt);
		Vector arCoef = wg.editor.getVector (differenceEquationARCoefSt);
		if (gain != Double.NEGATIVE_INFINITY && maCoef != null && arCoef != null) {
		    if (gain == 0) {
			wg.message.setVisible ("No valid Gain Factor");
			return;	
		    } 
		    if (maCoef.size() == 0 || arCoef.size() == 0) {
			wg.message.setVisible ("No valid coefficients");
			return;
		    }	
		    a = new double [arCoef.size()];
		    b = new double [maCoef.size()];
		    try {
			for (int i = 0; i < arCoef.size(); i++)
			    a[i] = (Double.valueOf((String)arCoef.elementAt(i))).doubleValue();
		    }
		    catch (IndexOutOfBoundsException ex) {}
		    try {
			for (int i = 0; i < maCoef.size(); i++)
			    b[i] = (Double.valueOf((String)maCoef.elementAt(i))).doubleValue();
		    }
		    catch (IndexOutOfBoundsException ex) {}
		    wg.params.saveParameter (this, maCoef, arCoef);
		    wg.params.saveParameter (this, gain);
		    proceedSignal.DifferenceEquation(a,b,gain,inv);
		} else return;
		break;
	    }
	   
	    case FILTERWITHGSECALIB: {
		twoPlots = false;
		int points = wg.editor.getInteger (numberOfPointsForFFTSt);
		if (points != Integer.MIN_VALUE) {
		    if (!wg.checkPoints (points)) {	
			wg.message.setVisible
			  ("Number of Points must be >= "+wg.params.find_next_nfft()+" and a power of 2 !");
			return;	
		    }
		    Spectrum spectrum = new Spectrum
			    (points, 1/wg.params.getSetupFrequency(), wg.CalVal);
		    if (wg.params.saveParameter (this, points))
			proceedSignal.BuildGSEResponseTrace
			    (points, spectrum.tr);
		    else proceedSignal = proceedSignal_before;    
		} else return;
		break;
	    }
	    
	    case BUTTERWORTH_BP: {
		double f_nyq = 0.5 / wg.frame.graph.t.t_samp;
		double flo = wg.editor.getDouble (butterworthBPLCSt);
		double fhi = wg.editor.getDouble (butterworthBPHCSt);
		int section = wg.editor.getInteger (butterworthSectionsSt);
		String zeroPhase = wg.editor.getSelection (butterworthFilSt);
		if (section != Integer.MIN_VALUE && flo != Double.NEGATIVE_INFINITY && 
		    fhi != Double.NEGATIVE_INFINITY) {
		    if ((flo <= 0) || (fhi < flo) || (flo > f_nyq) || (fhi > f_nyq)) {
			wg.message.setVisible ("No valid frequencies !");
			return;
		    }
		    if (section < 1 || section >= butterworthMAXSEC) {
			wg.message.setVisible ("No valid section !");
			return;
		    }
		    boolean b1 = wg.params.saveParameter (this, zeroPhase);
		    boolean b2 = wg.params.saveParameter (this, section);
		    boolean b3 = wg.params.saveParameter (this, flo, fhi);
		    if ( b1 || b2 || b3)
			 proceedSignal.butterworthBandPass (flo, fhi, section, zeroPhase);
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
		
	    case BUTTERWORTH_LP: {
	    	double f_nyq = 0.5 / wg.frame.graph.t.t_samp;
		double flo = wg.editor.getDouble (butterworthLPLPSt);
		int section = wg.editor.getInteger (butterworthSectionsSt);
		String zeroPhase = wg.editor.getSelection (butterworthFilSt);
		if (section != Integer.MIN_VALUE && flo != Double.NEGATIVE_INFINITY) {
		    if ((flo <= 0) || (flo > f_nyq)) {
			wg.message.setVisible ("No valid frequency !");
			return;
		    }
		    if (section < 1 || section >= butterworthMAXSEC) {
			wg.message.setVisible ("No valid section !");
			return;
		    }
		    boolean b1 = wg.params.saveParameter (this, zeroPhase);
		    boolean b2 = wg.params.saveParameter (this, section);
		    boolean b3 = wg.params.saveParameter (this, flo);
		    if ( b1 || b2 || b3)
			 proceedSignal.butterworthLowPass (flo, section, zeroPhase);
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
		
	    case BUTTERWORTH_HP: {
	    	double f_nyq = 0.5 / wg.frame.graph.t.t_samp;
		double fhi = wg.editor.getDouble (butterworthHPHPSt);
		int section = wg.editor.getInteger (butterworthSectionsSt);
		String zeroPhase = wg.editor.getSelection (butterworthFilSt);
		if (section != Integer.MIN_VALUE && fhi != Double.NEGATIVE_INFINITY) {
		    if ((fhi <= 0) || (fhi > f_nyq)) {
			wg.message.setVisible ("No valid frequeny !");
			return;
		    }
		    if (section < 1 || section >= butterworthMAXSEC) {
			wg.message.setVisible ("No valid section !");
			return;
		    }
		    boolean b1 = wg.params.saveParameter (this, zeroPhase);
		    boolean b2 = wg.params.saveParameter (this, section);
		    boolean b3 = wg.params.saveParameter (this, fhi);
		    if ( b1 || b2 || b3)
			 proceedSignal.butterworthHighPass (fhi, section, zeroPhase);
		    else proceedSignal = proceedSignal_before;
		} else return;
		break;
	    }
	    
	    case DELTAMODULATION: {
		twoPlots = false;
		double LSBvalue =  wg.editor.getDouble (modulationLSBSt);
		if (LSBvalue!= Double.NEGATIVE_INFINITY) {
	    	    if (LSBvalue < 0) {
			wg.message.setVisible ("No valid LSB !");
			return;
		    }
		    proceedSignal.deltaModulation (LSBvalue);
		    proceedSignal.MakeIntegrate ();
		    proceedSignal.barplot (proceedSignal.t_samp, 
		    	proceedSignal.t_samp,wg.params.getSetupPoints());
		    storeStore = false;
		} else return;
		break;
	    }
	    
	    case SIGMADELTAMODULATION: {
		twoPlots = false;
		double LSBvalue =  wg.editor.getDouble (modulationLSBSt);
		if (LSBvalue != Double.NEGATIVE_INFINITY) {
		    if (LSBvalue < 0) {
			wg.message.setVisible ("No valid LSB !");
			return;
		    }
		    proceedSignal.sigmaDeltaModulation (LSBvalue);
		    storeStore = false;
		} else return;
		break;
	    }
	}   // end of switch
	
	if (s.equals (okSt)) {
	    switch (id) {
		case DELTAMODULATION:
		    if (!storeStore) {
		        proceedSignal.deltaModulation (wg.editor.getDouble (modulationLSBSt));
		        proceedSignal.barplot
		    	(proceedSignal.t_samp,proceedSignal.t_samp,wg.params.getSetupPoints());
		        proceedSignal.theLabel = deltaModulationResultSt; 
		        proceedSignal.discr = true;
		    }
		    else 
		        storeStore = false;
		    break;
		case SIGMADELTAMODULATION:
		    if (storeStore)
			storeStore = false;
		    else
		        proceedSignal.sigmaDeltaModulation (wg.editor.getDouble (modulationLSBSt));
		    break;
	    }
	    wg.frame.graph.SetPlotTrace (proceedSignal);
	    wg.editor.setVisible(false);
	}
	else
	    wg.frame.graph.SetSecondPlotTrace(proceedSignal, twoPlots);
	    wg.actionPerformed (new ActionEvent (this, Event.ACTION_EVENT, "graphRepaint")); 
    }
 
 public boolean getInterpolationFactors (int ipol_fac)
 {
    int pow2,pow3,pow5;		    // powers of 2,3,5 in interpolation factor
    int conv_fac = ipol_fac;
	   
    //  number of divisions by 2
    pow2 = 2;
    no_it2 = 0;
    while(((float)conv_fac/(float)pow2) == (float)(conv_fac/pow2)) {
	pow2 *=2;
	no_it2 +=1;
    }
    if(no_it2 >0)
	conv_fac = 2*conv_fac/pow2;
	
    //  number of divisions by 3
    pow3 = 3;
    no_it3 = 0;
    while(((float)conv_fac/(float)pow3) == (float)(conv_fac/pow3)) {
	pow3 *=3;
	no_it3 +=1;
    }
    if(no_it3 > 0)
    conv_fac = 3*conv_fac/pow3;
    
    //  number of divisions by 5
    pow5 = 5;
    no_it5 = 0;
    while(((float)conv_fac/(float)pow5) == (float)(conv_fac/pow5)) {
	pow5 *=5;
	no_it5 +=1;
    }
    if (no_it5 >0)
    conv_fac = 5*conv_fac/pow5; 
    
    if( conv_fac != 1) return false;
    return true;
  }
  


}