//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.util.Vector;
import java.lang.*;

public class Settings implements Definitions 
{
    protected WinGroup wg;
    protected Vector defaultValues;
    protected Vector paramDescs;
    protected Vector paramClasses; 
    protected String title;
    protected String mainLabel;
    protected int id;
    
    // values that must be stored
    static protected int numberOfPointsForFFT = 0;
    static protected Vector fir2causFiles = null;
    static protected Vector gseCalibFiles = null;
    static protected Vector readDataDirsName = null;
    static protected Vector [] readDataDirsFiles = null;
    
    
    public int getId () { return id; }
     
    public Settings (WinGroup wg)
    {
	this.wg = wg;
	paramClasses = new Vector();
	defaultValues = new Vector();
	paramDescs = new Vector();
    }
    
    public Settings (Settings s)
    {
	wg = s.wg;
	id = s.id;
	title = (s.title == null) ? null : new String (s.title);
	mainLabel = (s.mainLabel == null) ? null : new String (s.mainLabel);
	defaultValues = new Vector (s.defaultValues.size());
	paramDescs = new Vector (s.paramDescs.size());
	paramClasses = new Vector (s.paramClasses.size());
	for (int i = 0; i < s.defaultValues.size(); i++)
	    defaultValues.addElement (s.defaultValues.elementAt (i));
	for (int i = 0; i < s.paramDescs.size(); i++)
	    paramDescs.addElement (s.paramDescs.elementAt (i));
	for (int i = 0; i < s.paramClasses.size(); i++)
	    paramClasses.addElement (s.paramClasses.elementAt (i));
    }
    
    void add (Class paramClass, String paramDesc, Object defaultValue)
    {
	paramClasses.addElement (paramClass);
	defaultValues.addElement (defaultValue);
	paramDescs.addElement (paramDesc);
    }
    
    void add (Class paramClass, String paramDesc)
    {
	paramClasses.addElement (paramClass);
	paramDescs.addElement (paramDesc);
	if (paramClass.getName().equals ("java.lang.Double"))
	    defaultValues.addElement (new Double (Double.NEGATIVE_INFINITY));
	if (paramClass.getName().equals ("java.lang.Integer"))
	    defaultValues.addElement (new Integer (Integer.MIN_VALUE));
	if (paramClass.getName().equals ("java.lang.String"))
	    defaultValues.addElement (new String (""));
    }
    
    int size ()
    {
	return (paramClasses.size());
    }
    
    public void removeAll()
    {
	paramClasses.removeAllElements();
	defaultValues.removeAllElements();
	paramDescs.removeAllElements();
	title = null;
	mainLabel = null;
    }
    
    public int newSettings (String s)
    {
      int d; 
	if (s.equals (sinusSt)) d = SINUS;
	else if (s.equals (constantSt)) d = CONSTANT;
	else if (s.equals (rickerSt)) d = RICKER;
	else if (s.equals (bruneSt)) d = BRUNE;
	else if (s.equals (mixedSt)) d = MIXED;
	else if (s.equals (chirpSt)) d = CHIRP;
	else if (s.equals (spikesSt)) d = SPIKES;
	else if (s.equals (seismometerDidrSt)) d = SEISMOMETER_DIDR;
	else if (s.equals (seismometerAsdrSt)) d = SEISMOMETER_ASDR;
	else if (s.equals (seismometerAsvrSt)) d = SEISMOMETER_ASVR;
	else if (s.equals (movingAverageSt)) d = MOVINGAVERAGE;
	else if (s.equals (interpolateSt)) d = INTERPOLATE;
	else if (s.equals (discretizationSt)) d = DISCRETIZATION;
	else if (s.equals (reconstructionSt)) d = RECONSTRUCTION;
	else if (s.equals (filterWithGseCalibSt)) d = FILTERWITHGSECALIB;
	else if (s.equals (decimateSt)) d = DECIMATE;
	else if (s.equals (deltaModulationSt)) d = DELTAMODULATION;
	else if (s.equals (sigmaDeltaModulationSt)) d = SIGMADELTAMODULATION;
	else if (s.equals (plainSummationSt)) d = PLAINSUMMATION;
	else if (s.equals (integrationSt)) d = INTEGRATION;
	else if (s.equals (differenceEquationSt)) d = DIFFERENCEEQUATION;
	else if (s.equals (plainADCQuantizedSt)) d = PLAINADCQUANTIZED;
	else if (s.equals (plainADCMantissaSt)) d = PLAINADCMANTISSA;
	else if (s.equals (plainADCExponentSt)) d = PLAINADCEXPONENT;
	else if (s.equals (plainADCErrorSt)) d = PLAINADCERROR;
	else if (s.equals (decimationFilteringSt)) d = DECIMATIONFILTERING;
	else if (s.equals (fir2causSt)) d = FIR2CAUS;
	else if (s.equals (openGseCalibrationFileSt) || s.equals(chooseGseCalibrationSt))
		d = GSECALIBRATIONFILE;
	else if (s.equals (dataTraceSt) || s.equals (openDataTraceSt)) d = DATATRACE;
	else if (s.equals (setupModifySt)) d = SETUPMODIFY;
	else if (s.equals (setupResetSt)) d = SETUPRESET;
	else if (s.equals (butterworthBPSt)) d = BUTTERWORTH_BP;
	else if (s.equals (butterworthLPSt)) d = BUTTERWORTH_LP;
	else if (s.equals (butterworthHPSt)) d = BUTTERWORTH_HP;
	else if (s.equals (differentiateSt)) d = DIFFERENTIATE;
	else if (s.equals (saveImageSt)) d = SAVEIMAGE;
	else if (s.equals (stepResponseSt)) d = STEPRESPONSE;
	else if (s.equals (impulseResponseSt)) d = IMPULSERESPONSE;
	else d  = -1;
	if (this.id != d || this.id == SETUPMODIFY || this.id == SETUPRESET) {
	    this.id = d;
	    removeAll();
	    Double w = new Double (0);
	    Integer i = new Integer (0);
	    String st = new String();
	    Vector v;
	    switch (id) {
		case SETUPMODIFY:
		    w = new Double (wg.params.getSetupFrequency ());
		    add (w.getClass(), samplingFrequencySt+" [Hz]", w);
		    i = new Integer (wg.params.getSetupPoints());
		    add (i.getClass(), setupPointsSt, i);
		    i = new Integer (wg.params.getSetupSpikePosition());
		    add (i.getClass(), setupSpikeSt, i);
		    i = new Integer (wg.params.getSetupStepPosition());
		    add (i.getClass(), setupStepSt, i);
		    title = setupTitelSt;
		    break;
		case SETUPRESET:
		    break;
		case SINUS: 
		    w = new Double (wg.params.getSinusAmplitude());
		    add (w.getClass(), amplitudeSt, w);
		    w = new Double (wg.params.getSinusFreqency());
		    add (w.getClass(), frequencySt, w);
		    w = new Double (wg.params.getSinusPhase());
		    add (w.getClass(), phaseSt, w);
		    title = sinusTitelSt;
		    mainLabel = sinusMainLabelSt;
		    break;
		case CONSTANT:
		    w = new Double (wg.params.getConstantAmplitude());
		    add (w.getClass(), amplitudeSt, w);
		    title = constantTitelSt;
		    mainLabel = constantMainLabelSt;
		    break;
		case RICKER:
		    w = new Double (wg.params.getRickerFrequency());
		    add (w.getClass(), frequencySt, w);
		    title = rickerTitelSt;
		    mainLabel = rickerMainLabelSt;
		    break;
		case BRUNE:
		    w = new Double (wg.params.getBruneRth());
		    add (w.getClass(), bruneRthSt, w);
		    w = new Double (wg.params.getBruneSigma());
		    add (w.getClass(), bruneSigmaSt, w);
		    w = new Double (wg.params.getBruneMu());
		    add (w.getClass(), bruneMuSt, w);
		    w = new Double (wg.params.getBruneVs());
		    add (w.getClass(), bruneVsSt, w);
		    w = new Double (wg.params.getBruneR());
		    add (w.getClass(), bruneRSt, w);
		    w = new Double (wg.params.getBruneZ());
		    add (w.getClass(), bruneZSt, w);
		    title = bruneTitelSt;
		    mainLabel = bruneMainLabelSt;
		    break;
		case MIXED:
		    w = new Double (wg.params.getMixedAmplitude());
		    add (w.getClass(), amplitudeSt, w);
		    w = new Double (wg.params.getMixedFrequency());
		    add (w.getClass(), frequencySt, w);
		    w = new Double (wg.params.getMixedPhase());
		    add (w.getClass(), phaseSt, w);
		    w = new Double (wg.params.getMixedB());
		    add (w.getClass(), mixedBSt, w);
		    w = new Double (wg.params.getMixedC());
		    add (w.getClass(), mixedCSt, w);
		    w = new Double (wg.params.getMixedD());
		    add (w.getClass(), mixedDSt, w);
		    title = mixedTitelSt;
		    mainLabel = mixedMainLabelSt;
		    break;
		case CHIRP: 
		    w = new Double (wg.params.getChirpAmplitude());
		    add (w.getClass(), amplitudeSt, w);
		    w = new Double (wg.params.getChirpFrequency1());
		    add (w.getClass(), chirpFrequency1St, w);
		    w = new Double (wg.params.getChirpPhase());
		    add (w.getClass(), phaseSt, w);
		    w = new Double (wg.params.getChirpFrequency2());
		    add (w.getClass(), chirpFrequency2St, w);
		    title = chirpTitelSt;
		    mainLabel = chirpMainLabelSt;
		    break;
		case SEISMOMETER_DIDR:
		case SEISMOMETER_ASDR:
		case SEISMOMETER_ASVR:
		    w = new Double (wg.params.getSeismometerFrequency());
		    add (w.getClass(), seismometerFrequencySt, w);
		    w = new Double (wg.params.getSeismometerDamping());
		    add (w.getClass(), seismometerDampingSt, w);
		    title = seismometerTitelSt;
		    mainLabel = seismometerMainLabelSt;
		    break;
		case SPIKES:
		    if (wg.params.getSpikesIndex() == null)
		    {
			v = new Vector();
			v.addElement (new Integer (0));
			add (v.getClass(), spikesPositionSt, v);
			add (v.getClass(), amplitudeSt, v);
		    }
		    else
		    {
			v = new Vector();
			v.addElement (new Integer (3));
			v.addElement (wg.params.getSpikesIndex());
			add (v.getClass(), spikesPositionSt, v);
			v = new Vector();
			v.addElement (new Integer (3));
			v.addElement (wg.params.getSpikesAmplitude());
			add (v.getClass(), amplitudeSt, v);
		    }
		    title = spikesTitelSt;
		    mainLabel = spikesMainLabelSt;
		    break;
		case MOVINGAVERAGE:
		    w = new Double (wg.params.getMovingAverageWindow());
		    add (w.getClass(), movingAverageWindowSt, w);
		    title = movingAverageTitelSt;
		    break;
		case INTERPOLATE:
		    i = new Integer (wg.params.getInterpolationFactor());
		    add (i.getClass(), interpolationFactorSt, i);
		    title = interpolateTitelSt;
		    break;
		case DISCRETIZATION:
		case RECONSTRUCTION:
		    w = new Double (wg.params.getDiscretizationFrequency());
		    add (w.getClass(), discretizationFrequencySt, w);
		    add (st.getClass(), samplingFrequencySt+" "+wg.params.getSetupFrequency()+" [Hz]");
		    title = (id == DISCRETIZATION) ? discretizationTitelSt : reconstructionTitelSt;
		    break;
		case FILTERWITHGSECALIB:
		    i = new Integer (wg.params.find_next_nfft());
		    add (i.getClass(), numberOfPointsForFFTSt, i);
		    title = filterWithGseCalibTitelSt;
		    break;
		case DECIMATE:
		    i = new Integer (wg.params.getDecimateFactor());
		    add (i.getClass(), decimateFactorSt, i);
		    i = new Integer (wg.params.getDecimateForce());
		    add (i.getClass(), decimateForceSt, i);
		    title = decimateTitelSt;
		    break;
		case DELTAMODULATION:
		    double x = wg.frame.graph.t.ymax/5;
		    x+=0.5*100;
		    int jj = (int)(x);
		    x = jj;
		    w = new Double(x / 100);
		    add (w.getClass(), modulationLSBSt, w);
		    title = deltaModulationTitelSt;
		    break;
		case SIGMADELTAMODULATION:
		    w = new Double(wg.frame.graph.t.ymax);
		    add (w.getClass(), modulationLSBSt, w);
		    title = sigmaDeltaModulationSt;
		    break;
		case DIFFERENTIATE:
		case PLAINSUMMATION:
		case INTEGRATION:
		    break;
		case DIFFERENCEEQUATION:
		    if (wg.params.getDiffEquMaCoef() == null)
		    {
			v = new Vector();
			v.addElement (new Integer(0));
			add (v.getClass(), differenceEquationARCoefSt, v);
			add (v.getClass(), differenceEquationMACoefSt, v);
		    }
		    else
		    {
			v = new Vector();
			v.addElement (new Integer(3));
			v.addElement (wg.params.getDiffEquArCoef());
			add (v.getClass(), differenceEquationARCoefSt, v);
			v = new Vector();
			v.addElement (new Integer(3));
			v.addElement (wg.params.getDiffEquMaCoef());
			add (v.getClass(), differenceEquationMACoefSt, v);
		    }
		    w = new Double (wg.params.getDiffEquGain());
		    add (w.getClass(), differenceEquationGainSt, w);
		    v = new Vector();
		    v.addElement (new Integer(2));
		    v.addElement (differenceEquationInvSt);
		    add (v.getClass(), differenceEquationInvSt, v);
		    title = differenceEquationTitelSt;
		    break;
		case PLAINADCQUANTIZED:
		case PLAINADCMANTISSA:
		case PLAINADCEXPONENT:
		case PLAINADCERROR:
		    add (st.getClass(), frequencySt);
		    w = new Double (wg.params.getPlainADCFrequency());
		    add (w.getClass(), plainADCFrequencySt + wg.params.getSetupFrequency() + " [Hz]", w);
		    add (st.getClass(), "");
		    add (st.getClass(), plainADCGain1St);
		    w = new Double (wg.params.getPlainADCGain());
		    add (w.getClass(), plainADCGain2St, w);
		    add (st.getClass(), "");
		    add (st.getClass(), plainADCMantissa1St);
		    i = new Integer (wg.params.getPlainADCMantissa());
		    add (i.getClass(), plainADCMantissa2St, i);
		    add (st.getClass(), "");
		    w = new Double (wg.params.getPlainADCLSB());
		    add (st.getClass(), "");
		    add (w.getClass(), plainADCLSBSt, w);
		    add (st.getClass(), "");
		    add (st.getClass(), plainADCExponent1St);
		    i = new Integer (wg.params.getPlainADCExponent());
		    add (i.getClass(), plainADCExponent2St, i);
		    add (st.getClass(), "");
		    title = s;
		    break;
		case DECIMATIONFILTERING:
		    v = new Vector();
		    v.addElement (new Integer(1));
		    v.addElement (decimationR2St);
		    v.addElement (decimationR4St);
		    v.addElement (decimationR10St);
		    v.addElement (decimationR16St);
		    add (v.getClass(), decimationRatioSt, v);
		    v = new Vector();
		    v.addElement (new Integer(1));
		    v.addElement (decimationFilteringResampOnSt);
		    v.addElement (decimationFilteringResampOffSt);
		    add (v.getClass(), decimationFilteringResampSt, v);
		    title = decimationFilteringSt;
		    break;
		case FIR2CAUS:
		    if (fir2causFiles == null)
			fir2causFiles = wg.ServerConnection.getFileList(wg.tutor.codeBase+wg.Dir_FIR, false);
		    v = new Vector();
		    v.addElement (new Integer(4));
		    v.addElement (fir2causFiles);
		    add (v.getClass(), fir2causCoorFilterFileSt, v);
		    w = new Double (wg.params.getFir2causFdig());
		    add (w.getClass(), fir2causFdigSt, w);
		    add (st.getClass(), fir2causFdig1St);
		    add (st.getClass(), "");
		    i = new Integer (wg.params.getFir2causZerosAppend());
		    add (i.getClass(), fir2causZerosAppendSt, i);
		    add (st.getClass(), "");
		    v = new Vector();
		    v.addElement (new Integer(1));
		    v.addElement (fir2causCorrectLinearPhaseYesSt);
		    v.addElement (fir2causCorrectLinearPhaseNoSt);
		    add (v.getClass(), fir2causCorrectLinearPhasesSt, v);
		    title = fir2causTitelSt;
		    break;
		case GSECALIBRATIONFILE:
		    if (gseCalibFiles == null)
			gseCalibFiles = wg.ServerConnection.getFileList(wg.tutor.codeBase+wg.Dir_calVal, false);
		    v = new Vector();
		    v.addElement (new Integer(4));
		    v.addElement (gseCalibFiles);
		    add (v.getClass(), chooseGseCalibrationFileSt, v);
		    title = openGseCalibrationFileTitleSt;
		    break;
		case DATATRACE:
		    if (readDataDirsName == null) {
			readDataDirsName = 
			    wg.ServerConnection.getFileList(wg.tutor.codeBase+wg.Dir_Data, true);
			readDataDirsFiles = new Vector [readDataDirsName.size()];
			for (int ii = 0; ii < readDataDirsName.size(); ii++) {
			    readDataDirsFiles[ii] = new Vector();
			    readDataDirsFiles[ii].addElement (readDataDirsName.elementAt(ii));
			    Vector vec = wg.ServerConnection.getFileList
				(wg.tutor.codeBase+wg.Dir_Data+readDataDirsName.elementAt(ii), false);
			    for (int j = 0; j < vec.size(); j++)
				readDataDirsFiles[ii].addElement((Object)vec.elementAt(j));
			} 
		    }
		    v = new Vector();
		    v.addElement (new Integer(6));
		    v.addElement (readDataDirsName);
		    add (v.getClass(), dataTraceFoldersSt, v);
		    v = new Vector();
		    v.addElement (new Integer (5));
		    v.addElement (readDataDirsFiles);
		    add (v.getClass(), dataTraceFilesSt, v);
		    w = new Double (wg.params.getDataTraceFrequency());
		    add (w.getClass(), dataTraceSamplingFrequencySt, w);
		    title = dataTraceTitleSt;
		    break;
		case BUTTERWORTH_BP:
		case BUTTERWORTH_LP:
		case BUTTERWORTH_HP:
		    switch (id) {
			case BUTTERWORTH_BP:
			    w = new Double (wg.params.getButterworthBPLowFrequncy());
			    add (w.getClass(), butterworthBPLCSt, w);
			    w = new Double (wg.params.getButterworthBPHighFrequncy());
			    add (w.getClass(), butterworthBPHCSt, w);
			    i = new Integer (wg.params.getButterworthBPSection());
			    break;
			case BUTTERWORTH_LP:
			    w = new Double (wg.params.getButterworthLPLowFrequncy());
			    add (w.getClass(), butterworthLPLPSt, w);
			    i = new Integer (wg.params.getButterworthLPSection());
			    break;
			case BUTTERWORTH_HP:
			    w = new Double (wg.params.getButterworthHPHighFrequncy());
			    add (w.getClass(), butterworthHPHPSt, w);
			    i = new Integer (wg.params.getButterworthHPSection());
			    break;
		    }
		    add (i.getClass(), butterworthSectionsSt, i);
		    v = new Vector();
		    v.addElement (new Integer(1));
		    v.addElement (butterworthForwardSt);
		    v.addElement (butterworthBackwardSt);
		    add (v.getClass(), butterworthFilSt, v);
		    title = s;
		    break;
		
		case SAVEIMAGE:
		    v = new Vector();
		    v.addElement (new Integer(7));
		    v.addElement (wg.frame.graph.getLabel());
		    add (v.getClass(), labelForImageSt, v);
		    break;
		
		case IMPULSERESPONSE:
		case STEPRESPONSE:
		    i = new Integer (wg.params.find_next_nfft());
		    add (i.getClass(), numberOfPointsForFFTSt, i);
		    break;
		default:
		    System.out.println ("Warning unknown Settings "+s);
	    }
	}
	return d;
    }
    
    // Used for SPECTRUM and FREQUENCYRESPONSE
    public void newSettings (int d)
    {
	if (this.id != d || this.id == SETUPMODIFY || this.id == SETUPRESET) {
	    this.id = d;
	    removeAll();
	    Integer i = new Integer (0);
	    i = new Integer (wg.params.find_next_nfft());
	    add (i.getClass(), numberOfPointsForFFTSt, i);
	}
    }
}
