//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class SettingsEditor extends WinFrame implements ItemListener
{
    protected Settings settings;
    protected Component[] components;
    protected CheckboxGroup[] chGroups;
    protected ActionListener listener;
    
    Vector listElements[];
    List liste;
    
    SettingsEditor (WinGroup w)
    {
	super (w, " ");
	settings = new Settings(w);
    }
     
    public void actionPerformed (ActionEvent e) {}
    
    public void newSettings (Settings set, ActionListener theListener)
    {
	if (settings.getId() == set.getId() && 
	    set.getId() != SETUPMODIFY && set.getId() != SETUPRESET)
	    return;
	removeAll();
	settings = new Settings (set);
	components = new Component [settings.size()];
	chGroups = new CheckboxGroup [settings.size()];
	listener = theListener;
    
	if (wg.index > 1) super.setTitle (settings.title + wg.index);
	else		  super.setTitle (settings.title);
	
	int row = 0;
	int col = 0;
	if (settings.mainLabel != null)
	    makeLabel (col, row++, 3, 1, settings.mainLabel);
	for (int i = 0; i < settings.size(); i++, col = 0) {
	    if (((Class)settings.paramClasses.elementAt(i)).getName().equals ("java.lang.String")) {
		makeLabel (col, row++, 3, 1, (String)settings.paramDescs.elementAt(i));
	    } 
	    else if (((Class)settings.paramClasses.elementAt(i)).getName().equals ("java.lang.Double")) {
		makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
		if (((Double)settings.defaultValues.elementAt(i)).doubleValue() == Double.NEGATIVE_INFINITY)
		    components[i] = new TextField ("");
		else
		    components[i] = new TextField (settings.defaultValues.elementAt(i).toString());
		makeComponent (++col, row++, 2,1, components[i]);
	    }
	    else if (((Class)settings.paramClasses.elementAt(i)).getName().equals ("java.lang.Integer")) {
		makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
		if (((Integer)settings.defaultValues.elementAt(i)).intValue() == Integer.MIN_VALUE)
		    components[i] = new TextField ("");
		else
		    components[i] = new TextField (settings.defaultValues.elementAt(i).toString());
		makeComponent (++col, row++, 2,1, components[i]);
	    }
	    else if (((Class)settings.paramClasses.elementAt(i)).getName().equals ("java.util.Vector")) {
		Vector vec = (Vector)settings.defaultValues.elementAt(i);
		int type = ((Integer)vec.elementAt (0)).intValue();
		int size = vec.size() - 1;
		switch (type) {
		    case 0:		    // empty TextArea
			makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
			components[i] = new TextArea (6, 12);
			makeComponent (++col, row++, 2,1, components[i]);
			break;
		    case 3:		    // filled TextArea
			StringBuffer text = new StringBuffer();
			makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
			components[i] = new TextArea (6, 12);
			Vector li = (Vector)vec.elementAt (1);
			for (int j = 0; j < li.size(); j++)
			{
			    text.append (li.elementAt (j));
			    text.append ("\n");
			}
			((TextArea)components[i]).setText (text.toString());
			makeComponent (++col, row++, 2,1, components[i]);
			break;	
		    case 7:		    // filled TextField
			makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
			String t = (String)vec.elementAt (1);
			components[i] = new TextField (t, t.length()+10);
			makeComponent (++col, row++, 2,1, components[i]);
			break;
    		    case 1:		    // CheckboxGroup
			makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
			Checkbox [] ch = new Checkbox [vec.size()];
			boolean fl = (size > 2) ? true : false;
			chGroups[i] = new CheckboxGroup ();
			for (int j = 0; j < size; j++) {
			    ch[j] = new Checkbox ((String)vec.elementAt(j+1),chGroups[i],(j==0)?true:false);
			    makeComponent (fl ? 1 : ++col, fl ? ++row: row, fl ? 2 : 1, 1, ch[j]);
			}
			row++;
			break;
		    case 2:		    // Checkbox
			components[i] = new Checkbox ((String)vec.elementAt(1));
			makeComponent (col, row++, 1,1,components[i]);
			break;
		    case 4:		    //List
		    case 6:		    //List with ItemListener
			makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
			components[i] = new List (6, false);
			li = (Vector)vec.elementAt (1);
			for (int j = 0; j < li.size(); j++)
			    ((List)components[i]).add ((String)li.elementAt (j));
			makeComponent (++col, row++, 2,1, components[i]);
			if (type == 6)
			    ((List)components[i]).addItemListener(this);
			break;
		    case 5:		    //List that is filled with ItemListener
			makeLabel (col,row, 1,1, (String)settings.paramDescs.elementAt(i));
			components[i] = liste = new List (10, false);
			listElements = (Vector[])vec.elementAt (1);
			makeComponent (++col, row++, 2,1, components[i]);
			break;
		}
	    }
	}
	makeCommandLine3B (row, listener);
    }
    
    public void itemStateChanged (ItemEvent e)
    { 
	liste.removeAll();
	int i = ((Integer)(e.getItem())).intValue();
	String dir = (String)listElements[i].elementAt (0);
	for (int j = 1; j < listElements[i].size(); j++)
	    liste.add ((String)listElements[i].elementAt (j));
    }
    
    public void showIt()
    {
	if (settings.size() == 0) {
	    listener.actionPerformed(new ActionEvent (this, Event.ACTION_EVENT,okSt));
	    return;
	}
	if (!isVisible()) {
	    setSize (100, 100);
	    setVisible (true);
	}
	setSize (getPreferredSize().width, getPreferredSize().height);
	setVisible (true);
    }    
        
    double getDouble (String what)
    { 
	double val = 0;
	for (int i = 0; i < settings.size(); i++)
	    if (settings.paramDescs.elementAt(i).equals (what)) {
		try { 
		    val = (Double.valueOf(((TextComponent)components[i]).getText())).doubleValue();
		}
		catch (NumberFormatException e) {
		    wg.message.setVisible ("'" + ERR(e.toString()) + "' is no valid value");
		    return Double.NEGATIVE_INFINITY;
		}
	    }
	return val;
    }
    
    int getInteger (String what)
    {
	int val = 0;
	for (int i = 0; i < settings.size(); i++)
	    if (settings.paramDescs.elementAt(i).equals (what)) {
		try {
		    val = (Integer.valueOf(((TextField)components[i]).getText())).intValue();
		}
		catch (NumberFormatException e) {
		    wg.message.setVisible ("'" + ERR(e.toString()) + "' is no valid value");
		    return Integer.MIN_VALUE;
		}
	    }
	return val;
    }
    
    public Vector getVector (String what)
    {
	for (int i = 0; i < settings.size(); i++)
	    if (settings.paramDescs.elementAt(i).equals (what))
		return StringToValues (((TextArea)components[i]).getText());
	return null;
    }
    
    public String getSelection (String what)
    {
	for (int i = 0; i < settings.size(); i++)
	    if (settings.paramDescs.elementAt(i).equals (what))
		return chGroups[i].getSelectedCheckbox().getLabel();
	return null;
    }  
    
    public String getSelectedItem (String what)
    {
	for (int i = 0; i < settings.size(); i++)
	    if (settings.paramDescs.elementAt(i).equals (what))
		return ((List)components[i]).getSelectedItem();
	return null;
    }
    
    public String getText (String what)
    {
	for (int i = 0; i < settings.size(); i++)
	    if (settings.paramDescs.elementAt(i).equals (what))
		return ((TextField)components[i]).getText();
	return null;
    }
    
    public boolean getState (String what)
    {
	for (int i = 0; i < settings.size(); i++)
	    if (settings.paramDescs.elementAt(i).equals (what))
		return ((Checkbox)components[i]).getState();
	return false;
    }  
} 