//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

// Build spectrum with given calibration values (from file or input)
//		  with nfft, t_samp selection

public class Spectrum extends PlotTrace{
  
  Spectrum (int points, double tsamp, CalibrationValues CV)
  {
    ComplexValue num, denom, s, t_om;   
    double delta_f, fr, om, imag;
    int i, ii;  

    nfft = points;
    t_samp = tsamp;
    tr = new double[nfft];
	
        delta_f = 1.0/(t_samp*nfft);
	for (i = 0; i <= nfft/2; i++)
	{	
	    fr = i*delta_f;
	    om = 2*Math.PI*fr;  
	    num = new ComplexValue(1.0, 0.0);
	    for (ii = 0; ii < CV.numOfZeros; ii++)
	    {
		s = new ComplexValue (0.0, om);
		s.Csub (CV.zeros[ii]);
		num.Cmul (s);
	    }
	    denom = new ComplexValue(1.0, 0.0);
	    for (ii = 0; ii < CV.numOfPoles; ii++)
	    {
		s = new ComplexValue (0.0, om);
		s.Csub (CV.poles[ii]);
		denom.Cmul (s);
	    }	

	    t_om = new ComplexValue(1.0, 0.0);
	    if ((denom.r != 0.0) || (denom.i != 0.0)) {
		num.Cdiv (denom);
		t_om = new ComplexValue (num.r, num.i);
	    }
	    t_om.r *= CV.scaleFak;
	    t_om.i *= CV.scaleFak;
	    if ((i < nfft/2) && (i > 0))
	    {
		tr[2*i] = t_om.r;
		tr[2*i+1] = t_om.i;
	    }
	    if (i == 0) tr[0] = t_om.r;
	    if (i == nfft/2) tr[1] = t_om.r;
	}
	
	// Calculate conjugate complex of input spectrum 
	for (i = 1; i < nfft/2; i++)
	{
	    imag = tr[2*i+1];
	    tr[2*i+1] = -imag;
	}        
    }  
  
}    