//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.lang.*;
import java.util.*;

public class TestSignal extends PlotTrace {
  
  public TestSignal (WinGroup wg, int param)
  {
     this.wg = wg;
     axis = 0;
     xLabel = "s";
     id = param;	
     t_samp = 1/wg.params.getSetupFrequency();
     nfft = wg.params.getSetupPoints();
     tr = new double[nfft];
     discr = false;
     switch (param) {
	case SINUS:		theLabel = sinusSt; break;
	case RICKER:		theLabel = rickerSt; break;
	case CONSTANT:		theLabel = constantSt; break;  
	case MIXED:		theLabel = mixedSt; break;
	case BRUNE:		theLabel = bruneSt; break;
	case CHIRP:		theLabel = chirpSt; break;
	case SEISMOMETER_DIDR:	theLabel = seismometerSt+": "+seismometerDidrSt; break;
	case SEISMOMETER_ASDR:	theLabel = seismometerSt+": "+seismometerAsdrSt; break;
        case SEISMOMETER_ASVR:	theLabel = seismometerSt+": "+seismometerAsvrSt; break;
	case SPIKES:		theLabel = spikesSt; break;
	case DATATRACE:		theLabel = dataTraceSt; break;
	case IMPULSE:
	case STEP:
      }
  }

  public void setParameter (double t_samp, int nfft)
  {
    this.t_samp = t_samp;
    this.nfft = nfft;
    tr = new double[nfft];
  }
 
  public void buildSignal ()
  {
    switch (id) {
	
	case DATATRACE: {
	    String u;
	    int p;
	    t_samp = 1/wg.params.getDataTraceFrequency();
	    String s = wg.params.getDataTraceFile();
	    if ((p = s.indexOf (wg.Dir_Data)) != -1)
	    {
		String n = s.substring (p);
		u = wg.tutor.codeBase+p;
	    }
	    else
		u = wg.tutor.codeBase+wg.Dir_Data+s;
	    if ((tr = wg.ServerConnection.ReadDataFile (u)) == null)
		break;
	    nfft = tr.length;
	    break;
	}
	
	case SINUS: {
	    double phase,omega,argu;
	    double time;
	    phase = wg.params.getSinusPhase()*2*Math.PI/360;
	    for (int i = 0; i < nfft;i++) {
		omega = 2 * Math.PI * wg.params.getSinusFreqency();
		time = i * t_samp;
		argu = omega * time + phase;
		tr[i] = wg.params.getSinusAmplitude() * Math.sin(argu);
	    }
	    break;
	}
	
	case CONSTANT: {
	    for (int i = 0; i < nfft; i++)
		tr[i] = wg.params.getConstantAmplitude();
	    break;
	}
	
	case RICKER: {
	    double argu,time;
	    for (int i = 0; i < nfft; i++) {
		time = (nfft/2-i)*t_samp;
		argu = Math.PI*Math.PI*time*time*
		       wg.params.getRickerFrequency()*wg.params.getRickerFrequency();
		tr[i] = (1.0 - 2*argu)*Math.exp(-argu);
	    }
	    break;
	}
  
	case BRUNE: {
	    double argu, time;
	    double sigma = wg.params.getBruneRth() * 1.0e5;    // bar -> Pa
	    double vs = wg.params.getBruneVs() * 1000;	    // km/sec -> m/sec
	    double r = wg.params.getBruneR() * 1000;	    // km-> m
	    double z = wg.params.getBruneZ() * 1000;	    // km -> m
	    for (int i = 0; i < nfft; i++) {
		time = i * t_samp;
		argu = -2.34*(vs/r)*time;
		tr[i] = 2.0*Math.exp(argu)*time*(wg.params.getBruneSigma()*
			sigma*vs*r/(wg.params.getBruneMu()*z));
	    }
	    break;
	}
	
	case MIXED: {
	    double phase,omega,argu,time;
	    phase = wg.params.getMixedPhase()*2.0*Math.PI/360.0;
	    for(int i = 0; i < nfft; i++) {
		omega = 2.0*Math.PI*wg.params.getMixedFrequency();
		time = i*t_samp;
		argu = omega*time + phase;
		tr[i] = wg.params.getMixedAmplitude()*Math.sin(argu)*
			Math.exp(-wg.params.getMixedB()*time)+
			wg.params.getMixedC() + wg.params.getMixedD()*time;
	    }
	    break;
	}
	
	case CHIRP: {
	    double omega,phase,argu,freq,time;
	    phase = wg.params.getChirpPhase() * 2.0*Math.PI/360.0;
	    for(int i = 0; i < nfft; i++) {
		freq = wg.params.getChirpFrequency1() + ((double)i/(nfft-1))*
		       (wg.params.getChirpFrequency2()-wg.params.getChirpFrequency1());
		omega = 2.0*Math.PI*freq;
		time = i*t_samp;
		argu = omega*time + phase;
		tr[i] = wg.params.getChirpAmplitude()* Math.sin(argu);
	    }
	    break;
        }
  
	case SEISMOMETER_DIDR: {    // displacement impulse response
	    // calculate filter coefficients  omega dash sub i */
	    double oms0,oms1;		    // filter coefficients  
	    double xk0,xk1;                 // k sub i dash
	    double a0,a1,a2,b0,bb1,b2;
	    double u0,u1,u2,v0,v1,v2;
	    double eps;
	    double time;
            oms0 = 0.0;
            oms1 = Math.tan(t_samp*Math.PI * wg.params.getSeismometerFrequency());

            //   k sub i dash 
            xk0 = 0.0;
            xk1 = 2.0*wg.params.getSeismometerDamping()*oms1;

            b0 = oms1*oms1 + xk1 +1.0;
            a0 = (oms0*oms0 + xk0 +1.0)/b0;
            a1 = 2.0*(oms0*oms0 -1.0)/b0;
            a2 = (oms0*oms0 - xk0 +1.0)/b0;
            bb1 = 2.0*(oms1*oms1 -1.0)/b0;
            b2 = (oms1*oms1 -xk1 +1.0)/b0;

            // filtering step 1  
            u0 = 0.0;
            u1 = 0.0;
            u2 = 0.0;
            v0 = 0.0;
            v1 = 0.0;
            v2 = 0.0;
            tr[0] = 1.0;
            for (int i = 0; i < nfft; i++) {
		u2 = u1;
                u1 = u0;
                u0 = tr[i];
                v2 = v1;
                v1 = v0;
                tr[i] = a0*u0 + a1*u1 +a2*u2 -bb1*v1 -b2*v2;
                v0 = tr[i];
             }
	     break;
	}    
	 
        case SEISMOMETER_ASDR:  {   // acceleration step -> displ. response 
				    // = damped harmonic oscillator 
	    double oms0,oms1;		    // filter coefficients
	    double eps;
	    double time;
	    oms0 = 2*Math.PI * wg.params.getSeismometerFrequency();
            eps = wg.params.getSeismometerDamping() * oms0;
            oms1 = Math.sqrt(oms0*oms0 - eps*eps);

            for (int i = 0; i < nfft; i++) {
		time = i * t_samp;
                tr[i] = - Math.exp(- eps * time)*(Math.cos(oms1*time) + (eps/oms1)*Math.sin(oms1*time));
            }
            break;
	}
	
        case SEISMOMETER_ASVR: {    // acceleration step -> velocity response
	    double oms0,oms1;		    // filter coefficients
	    double eps;
	    double time;
	    oms0 = 2*Math.PI * wg.params.getSeismometerFrequency();
            eps = wg.params.getSeismometerDamping() * oms0;
            oms1 = Math.sqrt(oms0*oms0 - eps*eps);

            tr[0] = 0.0;
            for (int i = 1; i < nfft; i++) {
		time = i * t_samp;
                tr[i] = 1.0/(Math.sqrt(1-wg.params.getSeismometerDamping()
			    *wg.params.getSeismometerDamping())*oms0)*Math.exp(- eps * time)*
			    Math.sin(Math.sqrt(1-wg.params.getSeismometerDamping()*
			    wg.params.getSeismometerDamping())*oms0*time);
            }
	    break;
        }
	
	case SPIKES: {
	    discr = true;
	    for (int i = 0; i < wg.params.getSpikesIndex().size(); i++)
		tr[(Integer.valueOf((String)wg.params.getSpikesIndex().elementAt(i))).intValue()] =
		(Double.valueOf((String)wg.params.getSpikesAmplitude().elementAt(i))).doubleValue();
	    break;
	}
	
	case IMPULSE: {
	    tr[wg.params.getSetupSpikePosition()] = 1;
	    break;
	}
	
	case STEP: {
	    for (int i = wg.params.getSetupStepPosition(); i < nfft; i++)
		tr[i] = 1; 
	    break;
	}
    }
 }  
 
}   
