//
// Elke Schmidtke, University of Potsdam
// eschmidt@geo.uni-potsdam.de

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
  Show the calibration values
*/

public class WinPolesZeros extends WinFrame
{
    static String Title = "Poles and Zeros ";
    TextArea ZerosRealPart;
    TextArea PolesImaginaryPart;
    TextArea PolesRealPart; 
    TextArea ZerosImaginaryPart;
    TextField ScaleFactorArea;
    String stPoles = "POLES";
    String stZeros = "ZEROS";
    String stReal = "real part";
    String stImag = "imaginary part"; 
    String stScale = "Scale Factor";
    double default_scale = 1.0;

    boolean ignorePolesWarning;
    
    public void setContinue (boolean cont)
    {
	ignorePolesWarning = cont;
    }
    
    public void actionPerformed (ActionEvent e)
    {
	String string = e.getActionCommand();
	if (string.equals (cancelSt)) setVisible(false);
	if (string.equals (resetSt)) Reset();
	if (string.equals (applySt) || string.equals (okSt))
	{
	    CalibrationValues cali = new CalibrationValues();
	    if (PolesZerosFromInput(cali))
	    {
		ignorePolesWarning = true;
		if (!cali.PolTest())
		    wg.warning.setVisible 
			("Warning","Poles should appear in complex conjugate pairs !", this);
		if (ignorePolesWarning)
		{
		    if (string.equals (okSt)) setVisible(false);
		    wg.CalVal.newCalibration(cali); 
		    wg.showPolZer.graph.G_Initalize_Poles_Zeros();
		    if (wg.showPolZer.isShowing()) wg.showPolZer.graph.repaint();
		    wg.actionPerformed 
			(new ActionEvent (this, Event.ACTION_EVENT, "acceptNewValues"));
		}	
	    }	
	}	
    }
  
    public void Reset()
    {
	ZerosRealPart.setText("");
	PolesImaginaryPart.setText("");
	PolesRealPart.setText(""); 
	ZerosImaginaryPart.setText("");
	ScaleFactorArea.setText(String.valueOf(default_scale));
	wg.CalVal.deleteCalibration();
    }
    
    protected boolean PolesZerosFromInput (CalibrationValues c)
    {	
      int i;
      Vector real;
      Vector imaginary;
      
	// Poles
	real = StringToValues (PolesRealPart.getText());
	imaginary = StringToValues (PolesImaginaryPart.getText());
	if ((c.numOfPoles = real.size()) == 0)
	{ 
	    wg.message.setVisible ("No valid input for poles");
	    return false;
	}	  
     	c.poles = new ComplexValue [c.numOfPoles];
	    for (i = 0; i < c.numOfPoles; i++)
		if (i <imaginary.size())
	    	    c.poles[i] = new ComplexValue
		    ((Float.valueOf((String)real.elementAt(i))).floatValue(),
		    (Float.valueOf((String)imaginary.elementAt(i))).floatValue());
		else
		    c.poles[i] = new ComplexValue
		    ((Float.valueOf((String)real.elementAt(i))).floatValue(), 0);	    
			
	// Zeros
	real = StringToValues (ZerosRealPart.getText());
	imaginary = StringToValues (ZerosImaginaryPart.getText());
	if ((c.numOfZeros = real.size()) > 0)
	{ 
	    c.zeros = new ComplexValue [c.numOfZeros];
	    for (i = 0; i < c.numOfZeros; i++)
		if (i < imaginary.size())
		    c.zeros[i] = new ComplexValue
		    ((Float.valueOf((String)real.elementAt(i))).floatValue(),
		     (Float.valueOf((String)imaginary.elementAt(i))).floatValue());
		else
		    c.zeros[i] = new ComplexValue
		    ((Float.valueOf((String)real.elementAt(i))).floatValue(), 0); 
	}
	
	// Scale Factor
	if ((ScaleFactorArea.getText()).equals (""))
	{
	    c.scaleFak = 1;
	    return true; 
	}   
	try
	{
	    c.scaleFak = (Float.valueOf(ScaleFactorArea.getText())).floatValue();
	}
	catch (NumberFormatException e)
	{	
	    wg.message.setVisible ("No valid scale Factor");
	    return false;	
	}	
	return true;
    }	
    
    public void newValues (CalibrationValues cali)
    {
      int i;
      StringBuffer werter,wertei;
	werter = new StringBuffer();
	wertei = new StringBuffer();
	for (i = 0; i < cali.numOfPoles; i++)
	{
	    werter.append (String.valueOf((float)cali.poles[i].r));
	    werter.append ("\n");
	    wertei.append (String.valueOf((float)cali.poles[i].i));
	    wertei.append ("\n");
	}    
	PolesRealPart.setText(werter.toString());   
	PolesImaginaryPart.setText(wertei.toString());
	werter = new StringBuffer();
	wertei = new StringBuffer();
    	for (i = 0; i < cali.numOfZeros; i++)
	{
	    werter.append (String.valueOf((float)cali.zeros[i].r));
	    werter.append ("\n");
	    wertei.append (String.valueOf((float)cali.zeros[i].i));
	    wertei.append ("\n");
	}  
	ZerosRealPart.setText(werter.toString());
	ZerosImaginaryPart.setText(wertei.toString());
	ScaleFactorArea.setText(String.valueOf((float)cali.scaleFak));
    }	
      
  // Constructor  
    WinPolesZeros (WinGroup w)
    {
	super (w, Title);
	width = 350;
	height= 450;
	
	// 1. line
	makeLabel (0,0, 2,1, stPoles);
	makeLabel (2,0, 1,1, stReal);
	makeLabel (3,0, 1,1, stImag);
	
	// 2. line
	PolesRealPart = new TextArea(6,12);
	makeComponent (2,1, 1,1, PolesRealPart);
	PolesImaginaryPart = new TextArea(6,12);
	makeComponent (3,1, 1,1, PolesImaginaryPart);
		
	// 3. line
	makeLabel (0,2, 2,1, stZeros);
	makeLabel (2,2, 1,1, stReal);
	makeLabel (3,2, 1,1, stImag);
	
	// 4. line
       	ZerosRealPart = new TextArea(6,12);
	makeComponent (2,3, 1,1, ZerosRealPart);
	ZerosImaginaryPart = new TextArea(6,12);
	makeComponent (3,3, 1,1, ZerosImaginaryPart);
		
	// 5. line
	makeLabel (0,4, 2,1, stScale);
	ScaleFactorArea = new TextField(String.valueOf(default_scale));
	makeComponent (2,4, 2,1, ScaleFactorArea);
		
	makeCommandLine4B (5, this);
	setSize (width, height);
   } 
} 